@@ -39,6 +39,9 @@ int Help()
     TEXTOUT("--Configuration=ConfigurationFile, -c ConfigurationFile");
     TEXTOUT("                    Use ConfigurationFile as configuration file");
     TEXTOUT("                    Default, it uses the one in the user data application file");
+    TEXTOUT("--PluginsConfiguration=PluginsConfigurationFile, -pc PluginsConfigurationFile");
+    TEXTOUT("                    Use PluginsConfigurationFile as plugins configuration file");
+    TEXTOUT("                    Default, it uses the struct in the configuration file or no plugins configured if not inside.");
     TEXTOUT("");
 
     TEXTOUT("Reporting Elements:");
@@ -111,6 +114,8 @@ int Help_Advanced()
     TEXTOUT("");
     TEXTOUT("--Force");
     TEXTOUT("                    Force to parse the file if registered in database");
+    TEXTOUT("--Async=yes, -as");
+    TEXTOUT("                    Analyze asynchronously the files, need to launch again the command to have the result");
     TEXTOUT("--Help=Ssl");
     TEXTOUT("                    More details about SSL specific options (e.g. for HTTPS or FTPS)");
     TEXTOUT("--Help=Ssh");
@@ -213,7 +218,7 @@ int Help_Xslt()
 //---------------------------------------------------------------------------
 int Version()
 {
-    TEXTOUT("MediaConch Command Line Interface 16.01");
+    TEXTOUT("MediaConch Command Line Interface 16.03");
 
     return CLI_RETURN_FINISH;
 }
