@@ -16,6 +16,7 @@
 
 #include <string>
 #include <vector>
+#include <map>
 #include <bitset>
 
 //---------------------------------------------------------------------------
@@ -48,15 +49,17 @@ public:
     //Config
     enum report
     {
-        report_MediaConch,
+        report_MediaConch = 0,
         report_MediaInfo,
         report_MediaTrace,
-        report_Max
+        report_MediaVeraPdf,
+        report_MediaDpfManager,
+        report_Max,
     };
 
     enum format
     {
-        format_Text,
+        format_Text = 0,
         format_Xml,         // XML corresponding to only one of MediaConch, MediaInfo, MediaTrace
         format_MaXml,       // MAXML, can contain one or more of MediaConch, MediaInfo, MediaTrace
         format_JsTree,
@@ -81,6 +84,12 @@ public:
         errorHttp_MAX          = -4,
     };
 
+    enum PluginType
+    {
+        PLUGIN_FORMAT = 0,
+        PLUGIN_MAX,
+    };
+
     struct ReportRes
     {
         std::string       report;
@@ -89,6 +98,13 @@ public:
         ReportRes() :     has_valid(false), valid(true) {}
     };
 
+    struct ValidateRes
+    {
+        std::string     file;
+        bool            valid;
+        ValidateRes() : valid(true) {}
+    };
+
     static const std::string display_xml_name;
     static const std::string display_maxml_name;
     static const std::string display_text_name;
@@ -106,17 +122,27 @@ public:
     int  analyze(const std::vector<std::string>& files, bool force_analyze = false);
     int  analyze(const std::string& file, bool& registered, bool force_analyze = false);
     int  is_done(const std::vector<std::string>& files, double& percent);
-    int  is_done(const std::string& file, double& percent);
+    int  is_done(const std::string& file, double& percent, report& report_kind);
+
+    void list(std::vector<std::string>& vec);
+    void file_from_id(int id, std::string& filename);
 
     // Output
     int  get_report(const std::bitset<report_Max>& Report, format f,
                     const std::vector<std::string>& files,
                     const std::vector<std::string>& policies_names,
                     const std::vector<std::string>& policies_contents,
+                    const std::map<std::string, std::string>& options,
                     MediaConchLib::ReportRes* result,
                     const std::string* display_name = NULL,
                     const std::string* display_content = NULL);
-    int  remove_report(const std::vector<std::string>& files);
+    int validate(MediaConchLib::report report, const std::vector<std::string>& files,
+                 const std::vector<std::string>& policies_names,
+                 const std::vector<std::string>& policies_contents,
+                 std::vector<ValidateRes*>& result);
+
+    //Clear
+    int remove_report(const std::vector<std::string>& files);
 
     // Implementation checker arguments
     void               set_implementation_schema_file(const std::string& file);
@@ -135,7 +161,11 @@ public:
     void               load_configuration();
     void               set_configuration_file(const std::string& file);
     const std::string& get_configuration_file() const;
+    void               load_plugins_configuration();
+    void               set_plugins_configuration_file(const std::string& file);
     void               set_compression_mode(compression compress);
+    int                get_ui_poll_request() const;
+    int                get_ui_database_path(std::string& path) const;
 
     bool ReportAndFormatCombination_IsValid(const std::vector<std::string>& files,
                                             const std::bitset<MediaConchLib::report_Max>& reports,
