@@ -25,7 +25,7 @@ namespace MediaConch {
 // RESTAPI
 //***************************************************************************
 
-const std::string RESTAPI::API_VERSION = "1.1";
+const std::string RESTAPI::API_VERSION = "1.3";
 
 //***************************************************************************
 // Constructor/Destructor
@@ -103,6 +103,25 @@ RESTAPI::Clear_Res::~Clear_Res()
     nok.clear();
 }
 
+//---------------------------------------------------------------------------
+RESTAPI::List_Res::~List_Res()
+{
+    for (size_t i = 0; i < files.size(); ++i)
+        delete files[i];
+    files.clear();
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::Validate_Res::~Validate_Res()
+{
+    for (size_t i = 0; i < ok.size(); ++i)
+        delete ok[i];
+    ok.clear();
+    for (size_t i = 0; i < nok.size(); ++i)
+        delete nok[i];
+    nok.clear();
+}
+
 //***************************************************************************
 // Request: to_str()
 //***************************************************************************
@@ -174,7 +193,11 @@ std::string RESTAPI::Report_Req::to_str() const
     out << "], policies_names_size: [" << policies_names.size();
     out << "], policies_contents_size: [" << policies_contents.size();
     out << "], display_name: [" << display_name;
-    out << "], display_content_length: [" << display_content.size() << "] ]";
+    out << "], display_content_length: [" << display_content.size();
+    out << "]";
+    if (has_verbosity)
+        out << ", verbosity: " << verbosity;
+    out << " ]";
     return out.str();
 }
 
@@ -210,6 +233,34 @@ std::string RESTAPI::Clear_Req::to_str() const
     return out.str();
 }
 
+//---------------------------------------------------------------------------
+std::string RESTAPI::Validate_Req::to_str() const
+{
+    std::stringstream out;
+
+    out << "[ids: [";
+    for (size_t i = 0; i < ids.size(); ++i)
+    {
+        if (i)
+            out << ", ";
+        out << ids[i];
+    }
+    RESTAPI api;
+    out << "], report: " << api.get_Report_string(report);
+    out << ", policies_names_size: [" << policies_names.size();
+    out << "], policies_contents_size: [" << policies_contents.size() << "] ]";
+    return out.str();
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::File_From_Id_Req::to_str() const
+{
+    std::stringstream out;
+
+    out << "[id: '" << id << "']";
+    return out.str();
+}
+
 //***************************************************************************
 // Result: to_str()
 //***************************************************************************
@@ -220,7 +271,7 @@ std::string RESTAPI::Analyze_Ok::to_str() const
     std::stringstream out;
 
     out << "{inId: " << inId;
-    out << ", outId: " << inId;
+    out << ", outId: " << outId;
     out << ", create: " << std::boolalpha << create << "}";
     return out.str();
 }
@@ -268,6 +319,11 @@ std::string RESTAPI::Status_Ok::to_str() const
     out << ", finished: " << std::boolalpha << finished;
     if (has_percent)
         out << ", done: " << done;
+    if (finished && has_tool)
+    {
+        RESTAPI api;
+        out << ", tool: " << api.get_Report_string(tool);
+    }
     out << "}";
     return out.str();
 }
@@ -413,6 +469,75 @@ std::string RESTAPI::Clear_Res::to_str() const
     return out.str();
 }
 
+//---------------------------------------------------------------------------
+std::string RESTAPI::List_Res::to_str() const
+{
+    std::stringstream out;
+
+    out << "[files: [";
+    for (size_t i = 0; i < files.size(); ++i)
+    {
+        if (i)
+            out << ", ";
+        out << "{file:" << files[i]->file;
+        out << ", id:" << files[i]->id << "}";
+    }
+    out << "] ]";
+    return out.str();
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::Validate_Ok::to_str() const
+{
+    std::stringstream out;
+
+    out << "{id: " << id << ", valid: " << std::boolalpha << valid << "}";
+    return out.str();
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::Validate_Nok::to_str() const
+{
+    std::stringstream out;
+
+    out << "{id: " << id;
+    RESTAPI api;
+    out << ", reason: " << api.get_Reason_string(error) << "}";
+    return out.str();
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::Validate_Res::to_str() const
+{
+    std::stringstream out;
+
+    out << "[ok: [";
+    for (size_t i = 0; i < ok.size(); ++i)
+    {
+        if (i)
+            out << ", ";
+        out << ok[i]->to_str();
+    }
+    out <<  "], nok: [";
+    for (size_t i = 0; i < nok.size(); ++i)
+    {
+        if (i)
+            out << ", ";
+        out << nok[i]->to_str();
+    }
+    out << "] ]";
+    return out.str();
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::File_From_Id_Res::to_str() const
+{
+    std::stringstream out;
+
+    out << "[file: '" << file << "']";
+    return out.str();
+}
+
 //***************************************************************************
 // Serialize: Request
 //***************************************************************************
@@ -423,10 +548,10 @@ std::string RESTAPI::serialize_analyze_req(Analyze_Req& req)
     Container::Value v, child;
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("args", serialize_analyze_args(req.args)));
+    child.obj["args"] = serialize_analyze_args(req.args);
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("ANALYZE", child));
+    v.obj["ANALYZE"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -457,21 +582,24 @@ std::string RESTAPI::serialize_report_req(Report_Req& req)
     Container::Value v, child;
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ids", serialize_ids(req.ids)));
-    child.obj.push_back(std::make_pair("reports", serialize_report_reports(req.reports)));
+    child.obj["ids"] = serialize_ids(req.ids);
+    child.obj["reports"] = serialize_report_reports(req.reports);
 
     if (req.policies_names.size())
-        child.obj.push_back(std::make_pair("policies_names", serialize_report_arr_str(req.policies_names)));
+        child.obj["policies_names"] = serialize_report_arr_str(req.policies_names);
     if (req.policies_contents.size())
-        child.obj.push_back(std::make_pair("policies_contents", serialize_report_arr_str(req.policies_contents)));
+        child.obj["policies_contents"] = serialize_report_arr_str(req.policies_contents);
 
     if (req.display_name.length())
-        child.obj.push_back(std::make_pair("display_name", serialize_report_string(req.display_name)));
+        child.obj["display_name"] = serialize_report_string(req.display_name);
     if (req.display_content.length())
-        child.obj.push_back(std::make_pair("display_content", serialize_report_string(req.display_content)));
+        child.obj["display_content"] = serialize_report_string(req.display_content);
+
+    if (req.has_verbosity)
+        child.obj["verbosity"] = serialize_report_int(req.verbosity);
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("REPORT", child));
+    v.obj["REPORT"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -485,10 +613,10 @@ std::string RESTAPI::serialize_retry_req(Retry_Req& req)
     Container::Value v, child;
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ids", serialize_ids(req.ids)));
+    child.obj["ids"] = serialize_ids(req.ids);
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("RETRY", child));
+    v.obj["RETRY"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -513,6 +641,59 @@ std::string RESTAPI::serialize_clear_req(Clear_Req& req)
     return ss.str();
 }
 
+//---------------------------------------------------------------------------
+std::string RESTAPI::serialize_list_req(List_Req&)
+{
+    return std::string();
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::serialize_validate_req(Validate_Req& req)
+{
+    Container::Value v, child, report;
+
+    child.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    child.obj["ids"] = serialize_ids(req.ids);
+
+    report.type = Container::Value::CONTAINER_TYPE_STRING;
+    report.s = get_Report_string(req.report);
+    child.obj["report"] = report;
+
+    if (req.policies_names.size())
+        child.obj["policies_names"] = serialize_report_arr_str(req.policies_names);
+    if (req.policies_contents.size())
+        child.obj["policies_contents"] = serialize_report_arr_str(req.policies_contents);
+
+    v.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    v.obj["VALIDATE"] = child;
+
+    std::string ret = model->serialize(v);
+    if (!ret.length())
+        error = model->get_error();
+    return ret;
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::serialize_file_from_id_req(File_From_Id_Req& req)
+{
+    Container::Value v, child, id;
+
+    child.type = Container::Value::CONTAINER_TYPE_OBJECT;
+
+    id.type = Container::Value::CONTAINER_TYPE_INTEGER;
+    id.l = req.id;
+
+    child.obj["id"] = id;
+
+    v.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    v.obj["FILE_FROM_ID"] = child;
+
+    std::string ret = model->serialize(v);
+    if (!ret.length())
+        error = model->get_error();
+    return ret;
+}
+
 //***************************************************************************
 // Serialize: Result
 //***************************************************************************
@@ -527,11 +708,11 @@ std::string RESTAPI::serialize_analyze_res(Analyze_Res& res)
         nok.array.push_back(serialize_generic_nok(res.nok[i]->id, res.nok[i]->error));
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ok", serialize_analyze_oks(res.ok)));
-    child.obj.push_back(std::make_pair("nok", nok));
+    child.obj["ok"] = serialize_analyze_oks(res.ok);
+    child.obj["nok"] = nok;
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("ANALYZE_RESULT", child));
+    v.obj["ANALYZE_RESULT"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -549,11 +730,11 @@ std::string RESTAPI::serialize_status_res(Status_Res& res)
         nok.array.push_back(serialize_generic_nok(res.nok[i]->id, res.nok[i]->error));
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ok", serialize_status_oks(res.ok)));
-    child.obj.push_back(std::make_pair("nok", nok));
+    child.obj["ok"] = serialize_status_oks(res.ok);
+    child.obj["nok"] = nok;
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("STATUS_RESULT", child));
+    v.obj["STATUS_RESULT"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -571,11 +752,11 @@ std::string RESTAPI::serialize_report_res(Report_Res& res)
         nok.array.push_back(serialize_generic_nok(res.nok[i]->id, res.nok[i]->error));
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ok", serialize_report_ok(res.ok)));
-    child.obj.push_back(std::make_pair("nok", nok));
+    child.obj["ok"] = serialize_report_ok(res.ok);
+    child.obj["nok"] = nok;
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("REPORT_RESULT", child));
+    v.obj["REPORT_RESULT"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -593,11 +774,11 @@ std::string RESTAPI::serialize_retry_res(Retry_Res& res)
         nok.array.push_back(serialize_generic_nok(res.nok[i]->id, res.nok[i]->error));
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ok", serialize_ids(res.ok)));
-    child.obj.push_back(std::make_pair("nok", nok));
+    child.obj["ok"] = serialize_ids(res.ok);
+    child.obj["nok"] = nok;
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("RETRY_RESULT", child));
+    v.obj["RETRY_RESULT"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -615,11 +796,78 @@ std::string RESTAPI::serialize_clear_res(Clear_Res& res)
         nok.array.push_back(serialize_generic_nok(res.nok[i]->id, res.nok[i]->error));
 
     child.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    child.obj.push_back(std::make_pair("ok", serialize_ids(res.ok)));
-    child.obj.push_back(std::make_pair("nok", nok));
+    child.obj["ok"] = serialize_ids(res.ok);
+    child.obj["nok"] = nok;
 
     v.type = Container::Value::CONTAINER_TYPE_OBJECT;
-    v.obj.push_back(std::make_pair("CLEAR_RESULT", child));
+    v.obj["CLEAR_RESULT"] = child;
+
+    std::string ret = model->serialize(v);
+    if (!ret.length())
+        error = model->get_error();
+    return ret;
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::serialize_list_res(List_Res& res)
+{
+    Container::Value v, child, files;
+
+    files.type = Container::Value::CONTAINER_TYPE_ARRAY;
+    for (size_t i = 0; i < res.files.size(); ++i)
+        files.array.push_back(serialize_list_file(res.files[i]->file, res.files[i]->id));
+
+    child.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    child.obj["files"] = files;
+
+    v.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    v.obj["LIST_RESULT"] = child;
+
+    std::string ret = model->serialize(v);
+    if (!ret.length())
+        error = model->get_error();
+    return ret;
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::serialize_validate_res(Validate_Res& res)
+{
+    Container::Value v, child, ok, nok;
+
+    ok.type = Container::Value::CONTAINER_TYPE_ARRAY;
+    for (size_t i = 0; i < res.ok.size(); ++i)
+        ok.array.push_back(serialize_validate_ok(res.ok[i]));
+
+    nok.type = Container::Value::CONTAINER_TYPE_ARRAY;
+    for (size_t i = 0; i < res.nok.size(); ++i)
+        nok.array.push_back(serialize_generic_nok(res.nok[i]->id, res.nok[i]->error));
+
+    child.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    child.obj["ok"] = ok;
+    child.obj["nok"] = nok;
+
+    v.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    v.obj["VALIDATE_RESULT"] = child;
+
+    std::string ret = model->serialize(v);
+    if (!ret.length())
+        error = model->get_error();
+    return ret;
+}
+
+//---------------------------------------------------------------------------
+std::string RESTAPI::serialize_file_from_id_res(File_From_Id_Res& res)
+{
+    Container::Value v, child, file;
+
+    file.type = Container::Value::CONTAINER_TYPE_STRING;
+    file.s = res.file;
+
+    child.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    child.obj["file"] = file;
+
+    v.type = Container::Value::CONTAINER_TYPE_OBJECT;
+    v.obj["FILE_FROM_ID_RESULT"] = child;
 
     std::string ret = model->serialize(v);
     if (!ret.length())
@@ -713,13 +961,14 @@ RESTAPI::Report_Req *RESTAPI::parse_report_req(std::string data)
     if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
         return NULL;
 
-    Container::Value *ids, *reports, *policies_names, *policies_contents, *display_name, *display_content;
+    Container::Value *ids, *reports, *policies_names, *policies_contents, *display_name, *display_content, *verbosity;
     ids = model->get_value_by_key(*child, "ids");
     reports = model->get_value_by_key(*child, "reports");
     policies_names = model->get_value_by_key(*child, "policies_names");
     policies_contents = model->get_value_by_key(*child, "policies_contents");
     display_name = model->get_value_by_key(*child, "display_name");
     display_content = model->get_value_by_key(*child, "display_content");
+    verbosity = model->get_value_by_key(*child, "verbosity");
 
     if (!ids || !reports || ids->type != Container::Value::CONTAINER_TYPE_ARRAY)
         return NULL;
@@ -759,6 +1008,12 @@ RESTAPI::Report_Req *RESTAPI::parse_report_req(std::string data)
     if (display_content && display_content->type == Container::Value::CONTAINER_TYPE_STRING)
         req->display_content = display_content->s;
 
+    if (verbosity && verbosity->type == Container::Value::CONTAINER_TYPE_INTEGER)
+    {
+        req->has_verbosity = true;
+        req->verbosity = verbosity->l;
+    }
+
     return req;
 }
 
@@ -834,6 +1089,109 @@ RESTAPI::Clear_Req *RESTAPI::parse_clear_req(std::string data)
     return req;
 }
 
+//---------------------------------------------------------------------------
+RESTAPI::List_Req *RESTAPI::parse_list_req(std::string data)
+{
+    Container::Value v, *child;
+
+    if (model->parse(data, v))
+    {
+        error = model->get_error();
+        return NULL;
+    }
+
+    child = model->get_value_by_key(v, "LIST");
+    if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return NULL;
+    List_Req *req = new List_Req;
+    return req;
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::Validate_Req *RESTAPI::parse_validate_req(std::string data)
+{
+    Container::Value v, *child;
+
+    if (model->parse(data, v))
+    {
+        error = model->get_error();
+        return NULL;
+    }
+
+    child = model->get_value_by_key(v, "VALIDATE");
+    if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return NULL;
+
+    Container::Value *ids, *report, *policies_names, *policies_contents;
+    ids = model->get_value_by_key(*child, "ids");
+    report = model->get_value_by_key(*child, "report");
+    policies_names = model->get_value_by_key(*child, "policies_names");
+    policies_contents = model->get_value_by_key(*child, "policies_contents");
+
+    if (!ids || !report || ids->type != Container::Value::CONTAINER_TYPE_ARRAY)
+        return NULL;
+
+    Validate_Req *req = new Validate_Req;
+    for (size_t i = 0; i < ids->array.size(); ++i)
+    {
+        Container::Value *id = &ids->array[i];
+
+        if (id->type != Container::Value::CONTAINER_TYPE_INTEGER)
+        {
+            delete req;
+            return NULL;
+        }
+        req->ids.push_back(id->l);
+    }
+
+    if (report->type != Container::Value::CONTAINER_TYPE_STRING)
+    {
+        delete req;
+        return NULL;
+    }
+    req->report = string_to_Report(report->s);
+
+    if (policies_names && policies_names->type == Container::Value::CONTAINER_TYPE_ARRAY)
+    {
+        for (size_t i = 0; i < policies_names->array.size(); ++i)
+            if (policies_names->array[i].type == Container::Value::CONTAINER_TYPE_STRING)
+                req->policies_names.push_back(policies_names->array[i].s);
+    }
+    if (policies_contents && policies_contents->type == Container::Value::CONTAINER_TYPE_ARRAY)
+    {
+        for (size_t i = 0; i < policies_contents->array.size(); ++i)
+            if (policies_contents->array[i].type == Container::Value::CONTAINER_TYPE_STRING)
+                req->policies_contents.push_back(policies_contents->array[i].s);
+    }
+
+    return req;
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::File_From_Id_Req *RESTAPI::parse_file_from_id_req(std::string data)
+{
+    Container::Value v, *child;
+
+    if (model->parse(data, v))
+    {
+        error = model->get_error();
+        return NULL;
+    }
+
+    child = model->get_value_by_key(v, "FILE_FROM_ID");
+    if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return NULL;
+
+    Container::Value *id;
+    id = model->get_value_by_key(*child, "id");
+    if (!id || id->type != Container::Value::CONTAINER_TYPE_INTEGER)
+        return NULL;
+
+    File_From_Id_Req *req = new File_From_Id_Req;
+    req->id = id->l;
+    return req;
+}
+
 //---------------------------------------------------------------------------
 RESTAPI::Analyze_Req *RESTAPI::parse_uri_analyze_req(const std::string&)
 {
@@ -915,6 +1273,27 @@ finish:
     return req;
 }
 
+//---------------------------------------------------------------------------
+RESTAPI::List_Req *RESTAPI::parse_uri_list_req(const std::string&)
+{
+    List_Req *req = new List_Req;
+    return req;
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::Validate_Req *RESTAPI::parse_uri_validate_req(const std::string&)
+{
+    Validate_Req *req = new Validate_Req;
+    return req;
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::File_From_Id_Req *RESTAPI::parse_uri_file_from_id_req(const std::string&)
+{
+    File_From_Id_Req *req = new File_From_Id_Req;
+    return req;
+}
+
 //---------------------------------------------------------------------------
 RESTAPI::Analyze_Res *RESTAPI::parse_analyze_res(std::string data)
 {
@@ -1189,6 +1568,121 @@ RESTAPI::Clear_Res *RESTAPI::parse_clear_res(std::string data)
     return res;
 }
 
+//---------------------------------------------------------------------------
+RESTAPI::List_Res *RESTAPI::parse_list_res(std::string data)
+{
+    Container::Value v, *child;
+
+    if (model->parse(data, v))
+    {
+        error = model->get_error();
+        return NULL;
+    }
+
+    child = model->get_value_by_key(v, "LIST_RESULT");
+    if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return NULL;
+
+    Container::Value *files;
+    files = model->get_value_by_key(*child, "files");
+
+    List_Res *res = new List_Res;
+    if (files->type != Container::Value::CONTAINER_TYPE_ARRAY)
+    {
+        delete res;
+        return NULL;
+    }
+
+    for (size_t i = 0; i < files->array.size(); ++i)
+    {
+        List_File *tmp = new List_File;
+
+        if (parse_list_file(&files->array[i], tmp->file, tmp->id))
+        {
+            delete res;
+            return NULL;
+        }
+        res->files.push_back(tmp);
+    }
+    return res;
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::Validate_Res *RESTAPI::parse_validate_res(std::string data)
+{
+    Container::Value v, *child;
+
+    if (model->parse(data, v))
+    {
+        error = model->get_error();
+        return NULL;
+    }
+
+    child = model->get_value_by_key(v, "VALIDATE_RESULT");
+    if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return NULL;
+
+    Container::Value *ok, *nok;
+    ok = model->get_value_by_key(*child, "ok");
+    nok = model->get_value_by_key(*child, "nok");
+
+    Validate_Res *res = new Validate_Res;
+    if (ok && parse_validate_ok(ok, res->ok))
+    {
+        delete res;
+        return NULL;
+    }
+
+    if (!nok)
+        return res;
+
+    if (nok->type != Container::Value::CONTAINER_TYPE_ARRAY)
+    {
+        delete res;
+        return NULL;
+    }
+
+    for (size_t i = 0; i < nok->array.size(); ++i)
+    {
+        Validate_Nok *tmp = new Validate_Nok;
+
+        if (parse_generic_nok(&nok->array[i], tmp->id, tmp->error))
+        {
+            delete res;
+            return NULL;
+        }
+        res->nok.push_back(tmp);
+    }
+    return res;
+}
+
+//---------------------------------------------------------------------------
+RESTAPI::File_From_Id_Res *RESTAPI::parse_file_from_id_res(std::string data)
+{
+    Container::Value v, *child;
+
+    if (model->parse(data, v))
+    {
+        error = model->get_error();
+        return NULL;
+    }
+
+    child = model->get_value_by_key(v, "FILE_FROM_ID_RESULT");
+    if (!child || child->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return NULL;
+
+    Container::Value *file;
+    file = model->get_value_by_key(*child, "file");
+
+    if (!file || file->type != Container::Value::CONTAINER_TYPE_STRING)
+        return NULL;
+
+    File_From_Id_Res *res = new File_From_Id_Res;
+    res->file = file->s;
+
+    return res;
+}
+
 //***************************************************************************
 // HELPER
 //***************************************************************************
@@ -1207,17 +1701,17 @@ Container::Value RESTAPI::serialize_analyze_args(std::vector<Analyze_Arg>& args)
 
         file.type = Container::Value::CONTAINER_TYPE_STRING;
         file.s = args[i].file;
-        arg.obj.push_back(std::make_pair("file", file));
+        arg.obj["file"] = file;
 
         id.type = Container::Value::CONTAINER_TYPE_INTEGER;
         id.l = args[i].id;
-        arg.obj.push_back(std::make_pair("id", id));
+        arg.obj["id"] = id;
 
         if (args[i].has_force_analyze)
         {
             force.type = Container::Value::CONTAINER_TYPE_BOOL;
             force.b = args[i].force_analyze;
-            arg.obj.push_back(std::make_pair("force", force));
+            arg.obj["force"] = force;
         }
 
         args_val.array.push_back(arg);
@@ -1275,6 +1769,17 @@ Container::Value RESTAPI::serialize_report_string(const std::string& reports)
     return str_val;
 }
 
+//---------------------------------------------------------------------------
+Container::Value RESTAPI::serialize_report_int(int val)
+{
+    Container::Value int_val;
+
+    int_val.type = Container::Value::CONTAINER_TYPE_INTEGER;
+    int_val.l = val;
+
+    return int_val;
+}
+
 //---------------------------------------------------------------------------
 Container::Value RESTAPI::serialize_report_arr_str(const std::vector<std::string>& reports)
 {
@@ -1302,11 +1807,11 @@ Container::Value RESTAPI::serialize_generic_nok(int id, Reason error)
 
     id_v.type = Container::Value::CONTAINER_TYPE_INTEGER;
     id_v.l = id;
-    nok.obj.push_back(std::make_pair("id", id_v));
+    nok.obj["id"] = id_v;
 
     error_v.type = Container::Value::CONTAINER_TYPE_STRING;
     error_v.s = get_Reason_string(error);
-    nok.obj.push_back(std::make_pair("error", error_v));
+    nok.obj["error"] = error_v;
     return nok;
 }
 
@@ -1328,15 +1833,15 @@ Container::Value RESTAPI::serialize_analyze_oks(std::vector<Analyze_Ok*>& array)
 
         in.type = Container::Value::CONTAINER_TYPE_INTEGER;
         in.l = array[i]->inId;
-        v.obj.push_back(std::make_pair("inId", in));
+        v.obj["inId"] = in;
 
         out.type = Container::Value::CONTAINER_TYPE_INTEGER;
         out.l = array[i]->outId;
-        v.obj.push_back(std::make_pair("outId", out));
+        v.obj["outId"] = out;
 
         create.type = Container::Value::CONTAINER_TYPE_BOOL;
         create.b = array[i]->create;
-        v.obj.push_back(std::make_pair("create", create));
+        v.obj["create"] = create;
 
         ok.array.push_back(v);
     }
@@ -1356,23 +1861,30 @@ Container::Value RESTAPI::serialize_status_oks(std::vector<Status_Ok*>& array)
         if (!array[i])
             continue;
 
-        Container::Value v, id, finished, done;
+        Container::Value v, id, finished, done, tool;
 
         v.type = Container::Value::CONTAINER_TYPE_OBJECT;
 
         id.type = Container::Value::CONTAINER_TYPE_INTEGER;
         id.l = array[i]->id;
-        v.obj.push_back(std::make_pair("id", id));
+        v.obj["id"] = id;
 
         finished.type = Container::Value::CONTAINER_TYPE_BOOL;
         finished.b = array[i]->finished;
-        v.obj.push_back(std::make_pair("finished", finished));
+        v.obj["finished"] = finished;
 
         if (array[i]->has_percent)
         {
             done.type = Container::Value::CONTAINER_TYPE_REAL;
             done.d = array[i]->done;
-            v.obj.push_back(std::make_pair("done", done));
+            v.obj["done"] = done;
+        }
+
+        if (array[i]->finished && array[i]->has_tool)
+        {
+            tool.type = Container::Value::CONTAINER_TYPE_INTEGER;
+            tool.l = array[i]->tool;
+            v.obj["tool"] = tool;
         }
 
         ok.array.push_back(v);
@@ -1393,18 +1905,57 @@ Container::Value RESTAPI::serialize_report_ok(Report_Ok& obj)
     {
         report.type = Container::Value::CONTAINER_TYPE_STRING;
         report.s = obj.report;
-        ok.obj.push_back(std::make_pair("report", report));
+        ok.obj["report"] = report;
     }
 
     if (obj.has_valid)
     {
         valid.type = Container::Value::CONTAINER_TYPE_BOOL;
         valid.b = obj.valid;
-        ok.obj.push_back(std::make_pair("valid", valid));
+        ok.obj["valid"] = valid;
     }
     return ok;
 }
 
+//---------------------------------------------------------------------------
+Container::Value RESTAPI::serialize_list_file(const std::string& filename, int id)
+{
+    Container::Value file, id_v, filename_v;
+
+    file.type = Container::Value::CONTAINER_TYPE_OBJECT;
+
+    id_v.type = Container::Value::CONTAINER_TYPE_INTEGER;
+    id_v.l = id;
+    file.obj["id"] = id_v;
+
+    filename_v.type = Container::Value::CONTAINER_TYPE_STRING;
+    filename_v.s = filename;
+    file.obj["file"] = filename_v;
+    return file;
+}
+
+//---------------------------------------------------------------------------
+Container::Value RESTAPI::serialize_validate_ok(Validate_Ok* obj)
+{
+    Container::Value ok;
+    ok.type = Container::Value::CONTAINER_TYPE_OBJECT;
+
+    if (!obj)
+        return ok;
+
+    Container::Value id, valid;
+
+    id.type = Container::Value::CONTAINER_TYPE_INTEGER;
+    id.l = obj->id;
+    ok.obj["id"] = id;
+
+    valid.type = Container::Value::CONTAINER_TYPE_BOOL;
+    valid.b = obj->valid;
+    ok.obj["valid"] = valid;
+
+    return ok;
+}
+
 //---------------------------------------------------------------------------
 int RESTAPI::parse_analyze_arg(Container::Value *v, std::vector<Analyze_Arg>& args)
 {
@@ -1531,11 +2082,12 @@ int RESTAPI::parse_status_ok(Container::Value *v, std::vector<Status_Ok*>& oks)
         if (obj->type != Container::Value::CONTAINER_TYPE_OBJECT)
             return -1;
 
-        Container::Value *id, *finished, *done;
+        Container::Value *id, *finished, *done, *tool;
 
         id = model->get_value_by_key(*obj, "id");
         finished = model->get_value_by_key(*obj, "finished");
         done = model->get_value_by_key(*obj, "done");
+        tool = model->get_value_by_key(*obj, "tool");
 
         if (!id || id->type != Container::Value::CONTAINER_TYPE_INTEGER ||
             !finished || finished->type != Container::Value::CONTAINER_TYPE_BOOL)
@@ -1560,6 +2112,14 @@ int RESTAPI::parse_status_ok(Container::Value *v, std::vector<Status_Ok*>& oks)
         else
             return -1;
 
+        if (tool && tool->type == Container::Value::CONTAINER_TYPE_INTEGER)
+        {
+            ok->has_tool = true;
+            ok->tool = (Report)tool->l;
+        }
+        else
+            ok->has_tool = false;
+
         oks.push_back(ok);
     }
 
@@ -1595,4 +2155,56 @@ int RESTAPI::parse_report_ok(Container::Value *v, Report_Ok& ok)
     return 0;
 }
 
+//---------------------------------------------------------------------------
+int RESTAPI::parse_list_file(Container::Value *v, std::string& file, int& id)
+{
+    if (v->type != Container::Value::CONTAINER_TYPE_OBJECT)
+        return -1;
+
+    Container::Value *id_v, *file_v;
+
+    file_v = model->get_value_by_key(*v, "file");
+    id_v = model->get_value_by_key(*v, "id");
+
+    if (!id_v || id_v->type != Container::Value::CONTAINER_TYPE_INTEGER ||
+        !file_v || file_v->type != Container::Value::CONTAINER_TYPE_STRING)
+        return -1;
+
+    id = id_v->l;
+    file = file_v->s;
+    return 0;
+}
+
+//---------------------------------------------------------------------------
+int RESTAPI::parse_validate_ok(Container::Value *v, std::vector<Validate_Ok*>& oks)
+{
+    if (v->type != Container::Value::CONTAINER_TYPE_ARRAY)
+        return -1;
+
+    for (size_t i = 0; i < v->array.size(); ++i)
+    {
+        Validate_Ok* ok = new Validate_Ok;
+        Container::Value *id, *valid;
+        id = model->get_value_by_key(v->array[i], "id");
+        valid = model->get_value_by_key(v->array[i], "valid");
+
+        if (id)
+        {
+            if (id->type != Container::Value::CONTAINER_TYPE_INTEGER)
+                return -1;
+            ok->id = id->l;
+        }
+
+        if (valid)
+        {
+            if (valid->type != Container::Value::CONTAINER_TYPE_BOOL)
+                return -1;
+            ok->valid = valid->b;
+        }
+        oks.push_back(ok);
+    }
+
+    return 0;
+}
+
 }
