@@ -14,6 +14,10 @@
 #include "Core.h"
 #include "Scheduler.h"
 #include "Queue.h"
+#include "Plugin.h"
+#include "PluginFormat.h"
+#include "PluginsManager.h"
+#include <ZenLib/Ztring.h>
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
@@ -32,6 +36,19 @@ namespace MediaConch {
     //---------------------------------------------------------------------------
     Scheduler::~Scheduler()
     {
+        queue->clear();
+        CS.Enter();
+        std::map<QueueElement*, QueueElement*>::iterator it = working.begin();
+        for (; it != working.end(); ++it)
+        {
+            if (it->first)
+            {
+                it->first->stop();
+                delete it->first;
+            }
+        }
+        working.clear();
+        CS.Leave();
     }
 
     //---------------------------------------------------------------------------
@@ -70,12 +87,16 @@ namespace MediaConch {
         if (!el)
             return;
 
+        if (another_work_to_do(el, MI) <= 0)
+            return;
+
         CS.Enter();
         core->register_file_to_database(el->filename, MI);
         std::map<QueueElement*, QueueElement*>::iterator it = working.find(el);
         if (it != working.end())
             working.erase(it);
         CS.Leave();
+
         run_element();
     }
 
@@ -87,6 +108,16 @@ namespace MediaConch {
         return size == 0;
     }
 
+    void Scheduler::get_elements(std::vector<std::string>& vec)
+    {
+        CS.Enter();
+        std::map<QueueElement*, QueueElement*>::iterator it = working.begin();
+        for (; it != working.end(); ++it)
+            if (it->first)
+                vec.push_back(it->first->filename);
+        CS.Leave();
+    }
+
     bool Scheduler::element_is_finished(const std::string& filename, double& percent_done)
     {
         bool ret = true;
@@ -128,4 +159,35 @@ namespace MediaConch {
         CS.Leave();
         return ret;
     }
+
+    int Scheduler::another_work_to_do(QueueElement *el, MediaInfoNameSpace::MediaInfoList* MI)
+    {
+        // Before registering, check the format
+        String format = MI->Get(0, Stream_General, 0, __T("Format"));
+        std::string format_str = ZenLib::Ztring(format).To_UTF8().c_str();
+
+        std::map<std::string, Plugin*> plugins = core->get_format_plugins();
+        if (plugins.find(format_str) != plugins.end() && plugins[format_str])
+        {
+            std::string error;
+            ((PluginFormat*)plugins[format_str])->set_file(el->filename);
+            plugins[format_str]->run(error);
+            const std::string& report = plugins[format_str]->get_report();
+            MediaConchLib::report report_kind = ((PluginFormat*)plugins[format_str])->get_report_kind();
+            core->register_file_to_database(el->filename, report, report_kind, MI);
+        }
+        else
+            return 1;
+
+        CS.Enter();
+        //core->register_plugin_report_to_database(el->filename, MI);
+        std::map<QueueElement*, QueueElement*>::iterator it = working.find(el);
+        if (it != working.end())
+            working.erase(it);
+        CS.Leave();
+
+        run_element();
+        return 0;
+    }
+
 }
