@@ -33,13 +33,13 @@ XsltRule::XsltRule(const XsltRule& r)
         return;
 
     this->title = r.title;
+    this->ope = r.ope;
     this->use_free_text = r.use_free_text;
     this->type = r.type;
     this->field = r.field;
     this->occurrence = r.occurrence;
-    this->ope = r.ope;
     this->value = r.value;
-    this->text = r.text;
+    this->test = r.test;
 }
 
 //***************************************************************************
@@ -67,7 +67,7 @@ XsltPolicy::~XsltPolicy()
 //---------------------------------------------------------------------------
 bool XsltPolicy::find_xslt_header(xmlNodePtr node)
 {
-    if (!node || node->type != XML_ELEMENT_NODE || !node->ns)
+    if (!node || node->type != XML_ELEMENT_NODE || !node->ns || !node->ns->prefix)
         return false;
 
     std::string ns("xsl");
@@ -138,7 +138,7 @@ bool XsltPolicy::find_call_template_xpath_node(xmlNodePtr node, std::string& xpa
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_call_template_free_text_node(xmlNodePtr node, XsltRule* rule, bool& valid)
+bool XsltPolicy::find_call_template_test_node(xmlNodePtr node, XsltRule* rule, bool& valid)
 {
     std::string def("call-template");
     if (!node->name || def.compare((const char*)node->name))
@@ -147,29 +147,33 @@ bool XsltPolicy::find_call_template_free_text_node(xmlNodePtr node, XsltRule* ru
     xmlChar *name = xmlGetNoNsProp(node, (const unsigned char*)"name");
     if (name == NULL)
         return false;
-    rule->ope = std::string((const char*)name);
+
+    std::string operation((const char*)name);
+    if (operation != "is_true")
+        return false;
+
+    rule->ope = operation;
 
     xmlNodePtr child = node->children;
     while (child)
     {
-        std::string xpath;
-        if (find_call_template_xpath_node(child, xpath))
-            if (xpath != rule->value)
+        if (find_call_template_xpath_node(child, rule->test))
+            if (rule->test != rule->value)
                 valid = false;
         child = child->next;
     }
 
     if (valid)
     {
-        rule->text = rule->value;
         rule->value = std::string();
         rule->use_free_text = true;
     }
+
     return true;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_choose_call_template_node(xmlNodePtr node, XsltRule* rule, bool& valid)
+bool XsltPolicy::find_choose_call_template_node(xmlNodePtr node, XsltRule* rule, const std::string& test, bool& valid)
 {
     std::string def("call-template");
     if (!node->name || def.compare((const char*)node->name))
@@ -183,6 +187,10 @@ bool XsltPolicy::find_choose_call_template_node(xmlNodePtr node, XsltRule* rule,
     }
     rule->ope = std::string((const char*)name);
 
+    valid = parse_test_for_rule(test, rule);
+    if (!valid)
+        return false;
+
     xmlNodePtr child = node->children;
     while (child)
     {
@@ -210,7 +218,20 @@ bool XsltPolicy::find_choose_call_template_node(xmlNodePtr node, XsltRule* rule,
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_choose_for_each_node(xmlNodePtr node, XsltRule* rule, bool& valid)
+bool XsltPolicy::validate_choose_for_each_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string def("text");
+    if (!def.compare((const char*)node->name))
+        return true;
+
+    return false;
+}
+
+//---------------------------------------------------------------------------
+bool XsltPolicy::find_choose_for_each_node(xmlNodePtr node, XsltRule* rule, const std::string& test, bool& valid)
 {
     std::string def("for-each");
     if (!node->name || def.compare((const char*)node->name))
@@ -219,24 +240,35 @@ bool XsltPolicy::find_choose_for_each_node(xmlNodePtr node, XsltRule* rule, bool
     xmlChar *select_str = xmlGetNoNsProp(node, (const unsigned char*)"select");
     if (select_str == NULL)
         return false;
-    std::string select((const char*)select_str);
 
-    valid = parse_test_for_rule(select, rule);
-    if (!valid)
-        return true;
+    if (test != (const char*)select_str)
+        return false;
 
     xmlNodePtr child = node->children;
     while (child)
     {
-        find_choose_call_template_node(child, rule, valid);
-        if (!valid)
-            return true;
+        if (!find_choose_call_template_node(child, rule, test, valid) || !valid)
+            if (!valid || !validate_choose_for_each_node(child))
+                return false;
         child = child->next;
     }
 
     return true;
 }
 
+//---------------------------------------------------------------------------
+bool XsltPolicy::validate_choose_when_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string def("text");
+    if (!def.compare((const char*)node->name))
+        return true;
+
+    return false;
+}
+
 //---------------------------------------------------------------------------
 bool XsltPolicy::find_choose_when_node(xmlNodePtr node, XsltRule* rule, bool& valid)
 {
@@ -247,20 +279,37 @@ bool XsltPolicy::find_choose_when_node(xmlNodePtr node, XsltRule* rule, bool& va
     xmlChar *test_str = xmlGetNoNsProp(node, (const unsigned char*)"test");
     if (test_str == NULL)
         return false;
-    std::string test((const char*)test_str);
 
-    valid = parse_test_for_rule(test, rule);
+    std::string test((const char*)test_str);
 
     xmlNodePtr child = node->children;
     while (child)
     {
-        find_choose_for_each_node(child, rule, valid);
+        if (!find_choose_for_each_node(child, rule, test, valid) || !valid)
+            if (!valid || !validate_choose_when_node(child))
+                return false;
         child = child->next;
     }
 
     return true;
 }
 
+//---------------------------------------------------------------------------
+bool XsltPolicy::validate_choose_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char*)node->name);
+    if (name == "text")
+        return true;
+
+    if (name == "otherwise")
+        return true;
+
+    return false;
+}
+
 //---------------------------------------------------------------------------
 bool XsltPolicy::find_choose_node(xmlNodePtr node, XsltRule* rule, bool& valid)
 {
@@ -271,9 +320,9 @@ bool XsltPolicy::find_choose_node(xmlNodePtr node, XsltRule* rule, bool& valid)
     xmlNodePtr child = node->children;
     while (child)
     {
-        find_choose_when_node(child, rule, valid);
-        if (!valid)
-            return true;
+        if (!find_choose_when_node(child, rule, valid) || !valid)
+            if (!valid || !validate_choose_node(child))
+                return false;
         child = child->next;
     }
 
@@ -322,6 +371,19 @@ bool XsltPolicy::find_context_attribute_node(xmlNodePtr node, XsltRule* rule)
     return false;
 }
 
+//---------------------------------------------------------------------------
+bool XsltPolicy::validate_check_context_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name == "text")
+        return true;
+
+    return false;
+}
+
 //---------------------------------------------------------------------------
 bool XsltPolicy::find_context_node(xmlNodePtr node, XsltRule* rule)
 {
@@ -332,7 +394,9 @@ bool XsltPolicy::find_context_node(xmlNodePtr node, XsltRule* rule)
     xmlNodePtr child = node->children;
     while (child)
     {
-        find_context_attribute_node(child, rule);
+        if (!find_context_attribute_node(child, rule))
+            if (!validate_check_context_node(child))
+                return false;
         child = child->next;
     }
     return true;
@@ -357,11 +421,23 @@ bool XsltPolicy::find_rule_title_node(xmlNodePtr node, std::string& t)
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_check_node(xmlNodePtr node)
+void XsltPolicy::validate_check_node(xmlNodePtr node, bool& valid)
+{
+    if (!node->name)
+        return;
+
+    std::string name((const char *)node->name);
+    if (name == "text")
+        return;
+    valid = false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::find_check_node(xmlNodePtr node)
 {
     std::string def("check");
     if (!node->name || def.compare((const char*)node->name))
-        return false;
+        return 1;
 
     bool valid = true;
     XsltRule *r = new XsltRule;
@@ -371,16 +447,19 @@ bool XsltPolicy::find_check_node(xmlNodePtr node)
         if (!find_rule_title_node(child, r->title))
             if (!find_context_node(child, r))
                 if (!find_choose_node(child, r, valid))
-                    find_call_template_free_text_node(child, r, valid);
+                    if (!find_call_template_test_node(child, r, valid))
+                        validate_check_node(child, valid);
         if (!valid)
             break;
         child = child->next;
     }
-    if (valid)
-        rules.push_back(r);
-    else
+    if (!valid)
+    {
         delete r;
-    return true;
+        return -1;
+    }
+    rules.push_back(r);
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -410,107 +489,249 @@ bool XsltPolicy::find_policychecks_name_node(xmlNodePtr node)
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_policychecks_node(xmlNodePtr node)
+bool XsltPolicy::validate_policychecks_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name == "text")
+        return true;
+    return false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::find_policychecks_node(xmlNodePtr node)
 {
     std::string def("policyChecks");
     if (!node->name || def.compare((const char*)node->name))
-        return false;
+        return 1;
 
     xmlNodePtr child = node->children;
+    int ret = 0;
     while (child)
     {
         if (!find_policychecks_name_node(child))
+        {
             if (!find_policychecks_description_node(child))
-                find_check_node(child);
+                ret = find_check_node(child);
+            else
+                ret = 0;
+        }
+        else
+            ret = 0;
+
+        if (ret < 0)
+            break;
+        else if (ret > 0 && !validate_policychecks_node(child))
+        {
+            ret = -1;
+            break;
+        }
+        else
+            ret = 0;
+        
         child = child->next;
     }
-    return true;
+    return ret;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_media_node(xmlNodePtr node)
+bool XsltPolicy::validate_media_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name == "text")
+        return true;
+    if (name == "attribute")
+        return true;
+    return false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::find_media_node(xmlNodePtr node)
 {
     std::string def("media");
     if (!node->name || def.compare((const char*)node->name))
-        return false;
+        return 1;
 
     xmlNodePtr child = node->children;
+    int ret = 0;
     while (child)
     {
-        find_policychecks_node(child);
+        ret = find_policychecks_node(child);
+        if (ret < 0)
+            break;
+        else if (ret > 0 && !validate_media_node(child))
+        {
+            ret = -1;
+            break;
+        }
+        else
+            ret = 0;
+
         child = child->next;
     }
-    return true;
+    return ret;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_for_each_node(xmlNodePtr node)
+bool XsltPolicy::validate_for_each_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name == "text")
+        return true;
+    return false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::find_for_each_node(xmlNodePtr node)
 {
     std::string def("for-each");
     if (!node->name || def.compare((const char*)node->name))
-        return false;
+        return 1;
 
     xmlNodePtr child = node->children;
+    int ret = 0;
     while (child)
     {
-        find_media_node(child);
+        ret = find_media_node(child);
+        if (ret < 0)
+            break;
+        else if (ret > 0 && !validate_for_each_node(child))
+        {
+            ret = -1;
+            break;
+        }
+        else
+            ret = 0;
         child = child->next;
     }
-    return true;
+    return ret;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_mediaconch_node(xmlNodePtr node)
+bool XsltPolicy::validate_mediaconch_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name == "text")
+        return true;
+    if (name == "attribute")
+        return true;
+    return false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::find_mediaconch_node(xmlNodePtr node)
 {
     std::string def("MediaConch");
     if (!node->name || def.compare((const char*)node->name))
-        return false;
+        return 1;
 
     xmlNodePtr child = node->children;
+    int ret = 0;
     while (child)
     {
-        find_for_each_node(child);
+        ret = find_for_each_node(child);
+        if (ret < 0)
+            break;
+        else if (ret > 0 && !validate_mediaconch_node(child))
+        {
+            ret = -1;
+            break;
+        }
+        else
+            ret = 0;
         child = child->next;
     }
+    return ret;
+}
+
+//---------------------------------------------------------------------------
+bool XsltPolicy::validate_template_match_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name != "text")
+        return false;
     return true;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_template_match_node(xmlNodePtr node)
+int XsltPolicy::find_template_match_node(xmlNodePtr node)
 {
-    xmlChar *name = xmlGetNoNsProp(node, (const unsigned char*)"match");
-    if (name == NULL)
-        return false;
+    xmlChar *match = xmlGetNoNsProp(node, (const unsigned char*)"match");
+    
+    if (match == NULL || std::string((const char*)match) != "ma:MediaArea")
+        return 1;
 
     xmlNodePtr child = node->children;
+    int ret = 0;
     while (child)
     {
-        find_mediaconch_node(child);
+        ret = find_mediaconch_node(child);
+        if (ret < 0)
+            break;
+        else if (ret > 0 && !validate_template_match_node(child))
+        {
+            ret = -1;
+            break;
+        }
+        else
+            ret = 0;
         child = child->next;
     }
-    return true;
+    return ret;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_template_name_node(xmlNodePtr node)
+int XsltPolicy::validate_template_match_name(xmlNodePtr node)
 {
-    xmlChar *name = xmlGetNoNsProp(node, (const unsigned char*)"name");
-    if (name == NULL)
-        return false;
-    //TODO: operation
-    return true;
+    xmlChar *name_s = xmlGetNoNsProp(node, (const unsigned char*)"name");
+    if (!name_s)
+        return -1;
+
+    std::string name((const char*)name_s);
+
+    if (operator_exists(name))
+        return 0;
+    return -1;
 }
 
 //---------------------------------------------------------------------------
-bool XsltPolicy::find_template_node(xmlNodePtr node)
+bool XsltPolicy::valid_root_child_name_node(xmlNodePtr node)
+{
+    if (!node->name)
+        return false;
+
+    std::string name((const char *)node->name);
+    if (name == "template" || name == "output" || name == "text")
+        return true;
+
+    return false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::find_template_node(xmlNodePtr node)
 {
     std::string def("template");
     if (!node->name || def.compare((const char*)node->name))
-        return false;
+        return 1;
 
-    if (find_template_match_node(node))
-        return true;
-    return find_template_name_node(node);
+    int ret = find_template_match_node(node);
+    if (ret > 0)
+        ret = validate_template_match_name(node);
+    return ret;
 }
 
 //---------------------------------------------------------------------------
@@ -555,13 +776,24 @@ int XsltPolicy::import_schema_from_doc(const std::string& filename, xmlDocPtr do
         if (end_index != std::string::npos)
             title = title.substr(0, end_index);
     }
+
+    int ret = 0;
     while (child)
     {
-        find_template_node(child);
+        ret = find_template_node(child);
+        if (ret < 0)
+            break;
+        else if (ret > 0 && !valid_root_child_name_node(child))
+        {
+            ret = -1;
+            break;
+        }
+        else
+            ret = 0;
         child = child->next;
     }
     xmlSetGenericErrorFunc(NULL, NULL);
-    return 0;
+    return ret;
 }
 
 //---------------------------------------------------------------------------
@@ -601,29 +833,41 @@ xmlNsPtr XsltPolicy::create_namespace_xsi(xmlNodePtr node)
 //---------------------------------------------------------------------------
 void XsltPolicy::create_test_from_rule(XsltRule *rule, std::string& xpath)
 {
-    std::stringstream ss;
-
-    ss << "mi:MediaInfo/mi:track[@type='";
-
-    if (!rule->type.length())
+    if (rule->ope == "is_true")
         return;
-    ss << rule->type << "']";
 
-    ss << "[";
-    if (rule->occurrence < 0)
-        ss << "*";
-    else
-        ss << rule->occurrence;
-    ss << "]";
-
-    if (!rule->field.length())
+    if (!rule->use_free_text)
     {
-        xpath = ss.str();
-        return;
-    }
-    ss << "/mi:" << rule->field;
+        std::stringstream ss;
 
-    xpath = ss.str();
+        ss << "mi:MediaInfo/mi:track[@type='";
+
+        if (rule->type.length())
+            ss << rule->type;
+        ss << "']";
+
+        ss << "[";
+        if (rule->occurrence < 0)
+            ss << "*";
+        else
+            ss << rule->occurrence;
+        ss << "]";
+
+        if (rule->field.length())
+            ss << "/mi:" << rule->field;
+
+        xpath = ss.str();
+    }
+    else
+        xpath = rule->test;
+}
+
+//---------------------------------------------------------------------------
+bool XsltPolicy::parse_test_for_rule_free_text(const std::string& test, XsltRule *rule)
+{
+    rule->use_free_text = true;
+    rule->test = test;
+    return true;
 }
 
 //---------------------------------------------------------------------------
@@ -633,22 +877,26 @@ bool XsltPolicy::parse_test_for_rule(const std::string& test, XsltRule *rule)
     std::string find("mi:MediaInfo/mi:track[@type='");
     size_t pos = 0;
     size_t end = 0;
-    // type
+
+    rule->use_free_text = false;
+
+    // Type
     if ((pos = ss.find(find)) != 0)
-        return false;
+        return parse_test_for_rule_free_text(test, rule);
 
     pos += find.length();
     find = "']";
     if ((end = ss.find(find, pos)) == std::string::npos)
-        return false;
+        return parse_test_for_rule_free_text(test, rule);
 
     rule->type = ss.substr(pos, end - pos);
     if (!rule->type.length())
-        return false;
+        return parse_test_for_rule_free_text(test, rule);
 
     end += find.length();
+    // Field and Occurrence are mandatory
     if (end == ss.length())
-        return true;
+        return parse_test_for_rule_free_text(test, rule);
 
     // occurrence
     pos = end;
@@ -658,7 +906,7 @@ bool XsltPolicy::parse_test_for_rule(const std::string& test, XsltRule *rule)
         pos += find.length();
         find = "]";
         if ((end = ss.find("]", pos)) == std::string::npos)
-            return false;
+            return parse_test_for_rule_free_text(test, rule);
 
         std::string occurrence = ss.substr(pos, end - pos);
         if (occurrence == "*")
@@ -670,23 +918,30 @@ bool XsltPolicy::parse_test_for_rule(const std::string& test, XsltRule *rule)
     else
         rule->occurrence = -1;
 
+    // Field is empty
     if (end == ss.length())
-        return true;
+    {
+        if (rule->field.length())
+            return parse_test_for_rule_free_text(test, rule);
+        else
+            return true;
+    }
 
-    // field
+    // Field
     pos = end;
     find = "/mi:";
     if ((pos = ss.find(find, end)) != end)
-        return false;
+        return parse_test_for_rule_free_text(test, rule);
 
     pos += find.length();
     end = ss.length();
 
     std::string field = ss.substr(pos, end - pos);
     if (!field.length())
-        return false;
-    rule->field = field;
+        return parse_test_for_rule_free_text(test, rule);
 
+    if (field != rule->field)
+        return false;
     return true;
 }
 
@@ -1051,33 +1306,20 @@ void XsltPolicy::write_operators(xmlNodePtr node)
     write_operator_contains_string(node);
 }
 
-//---------------------------------------------------------------------------
-void XsltPolicy::write_check_call_template_field_child(xmlNodePtr node, XsltRule *rule)
-{
-    if (rule->use_free_text || !rule->field.length() ||
-        (rule->ope != "is_greater_than" && rule->ope != "is_less_than" &&
-         rule->ope != "is_greater_or_equal_than" && rule->ope != "is_less_or_equal_than" &&
-         rule->ope != "exists" && rule->ope != "does_not_exist"))
-        return;
-
-    xmlNodePtr child = xmlNewNode(NULL, (const xmlChar *)"with-param");
-    child->ns = node->ns;
-    xmlNewProp(child, (const xmlChar *)"name", (const xmlChar *)"field");
-    xmlNodeSetContent(child, (const xmlChar *)rule->field.c_str());
-
-    xmlAddChild(node, child);
-}
-
 //---------------------------------------------------------------------------
 void XsltPolicy::write_check_call_template_value_child(xmlNodePtr node, XsltRule *rule)
 {
-    if (rule->use_free_text || !rule->value.length())
+    if (rule->ope == "is_true" || !operator_need_value(rule->ope))
         return;
 
+    const char* value = "";
+    if (rule->value.length())
+        value = rule->value.c_str();
+
     xmlNodePtr child = xmlNewNode(NULL, (const xmlChar *)"with-param");
     child->ns = node->ns;
     xmlNewProp(child, (const xmlChar *)"name", (const xmlChar *)"value");
-    xmlNodeSetContent(child, (const xmlChar *)rule->value.c_str());
+    xmlNodeSetContent(child, (const xmlChar *)value);
 
     xmlAddChild(node, child);
 }
@@ -1085,12 +1327,9 @@ void XsltPolicy::write_check_call_template_value_child(xmlNodePtr node, XsltRule
 //---------------------------------------------------------------------------
 void XsltPolicy::write_check_call_template_xpath_child(xmlNodePtr node, XsltRule *rule)
 {
-    if (rule->use_free_text && !rule->text.length())
-        return;
-
     const char *xpath_str = ".";
-    if (rule->use_free_text)
-        xpath_str = rule->text.c_str();
+    if (rule->ope == "is_true" && rule->test.length())
+        xpath_str = rule->test.c_str();
 
     xmlNodePtr child = xmlNewNode(NULL, (const xmlChar *)"with-param");
     child->ns = node->ns;
@@ -1107,17 +1346,11 @@ void XsltPolicy::write_check_call_template_child(xmlNodePtr node, XsltRule *rule
     child->ns = create_namespace_xsl(NULL);
     xmlAddChild(node, child);
 
-    const char *operat = NULL;
-
-    if (!rule->use_free_text)
-        operat = rule->ope.c_str();
-    else
-        operat = "is_true";
+    const char *operat = rule->ope.c_str();
 
     xmlNewProp(child, (const xmlChar *)"name", (const xmlChar *)operat);
     write_check_call_template_xpath_child(child, rule);
     write_check_call_template_value_child(child, rule);
-    write_check_call_template_field_child(child, rule);
 }
 
 //---------------------------------------------------------------------------
@@ -1168,7 +1401,7 @@ void XsltPolicy::write_check_otherwise_child(xmlNodePtr node, XsltRule *rule)
 //---------------------------------------------------------------------------
 void XsltPolicy::write_check_choose_child(xmlNodePtr node, XsltRule *rule)
 {
-    if (rule->use_free_text)
+    if (rule->ope == "is_true")
     {
         write_check_call_template_child(node, rule);
         return;
@@ -1181,16 +1414,27 @@ void XsltPolicy::write_check_choose_child(xmlNodePtr node, XsltRule *rule)
     write_check_otherwise_child(child, rule);
 }
 
+//---------------------------------------------------------------------------
+void XsltPolicy::write_check_context_value_is_true_child(xmlNodePtr node, XsltRule *rule)
+{
+    const char *value = "";
+    if (rule->test.length())
+        value = rule->test.c_str();
+
+    xmlNodePtr child = xmlNewNode(NULL, (const xmlChar *)"attribute");
+    child->ns = create_namespace_xsl(NULL);
+    xmlNewProp(child, (const xmlChar *)"name", (const xmlChar *)"value");
+    xmlNodeSetContent(child, (const xmlChar *)value);
+
+    xmlAddChild(node, child);
+}
+
 //---------------------------------------------------------------------------
 void XsltPolicy::write_check_context_value_child(xmlNodePtr node, XsltRule *rule)
 {
-    const char *value = NULL;
-    if (!rule->use_free_text && rule->value.length())
+    const char *value = "";
+    if (rule->value.length())
         value = rule->value.c_str();
-    else if (rule->use_free_text && rule->text.length())
-        value = rule->text.c_str();
-    else
-        return;
 
     xmlNodePtr child = xmlNewNode(NULL, (const xmlChar *)"attribute");
     child->ns = create_namespace_xsl(NULL);
@@ -1222,7 +1466,10 @@ void XsltPolicy::write_check_context_child(xmlNodePtr node, XsltRule *rule)
     xmlAddChild(node, child);
     
     write_check_context_field_child(child, rule);
-    write_check_context_value_child(child, rule);
+    if (operator_need_value(rule->ope))
+        write_check_context_value_child(child, rule);
+    else if (rule->ope == "is_true")
+        write_check_context_value_is_true_child(child, rule);
 }
 
 //---------------------------------------------------------------------------
@@ -1416,4 +1663,62 @@ xmlDocPtr XsltPolicy::create_doc()
     return doc;
 }
 
+//---------------------------------------------------------------------------
+bool XsltPolicy::operator_need_value(const std::string& ope)
+{
+    std::string tmp;
+    if (get_operator_value(ope, tmp) < 0)
+        return false;
+
+    return true;
+}
+
+//---------------------------------------------------------------------------
+bool XsltPolicy::operator_exists(const std::string& ope)
+{
+    std::list<std::string>::const_iterator it = Policies::existing_xsltOperator.begin();
+
+    for (; it != Policies::existing_xsltOperator.end(); ++it)
+    {
+        if (*it == ope)
+            return true;
+    }
+
+    return false;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::get_operator_value(const std::string& ope, std::string& value)
+{
+    std::list<Policies::validatorType>::const_iterator it = Policies::existing_validator.begin();
+
+    for (; it != Policies::existing_validator.end(); ++it)
+    {
+        if (it->name == ope)
+        {
+            value = it->value;
+            return 0;
+        }
+    }
+
+    return -1;
+}
+
+//---------------------------------------------------------------------------
+int XsltPolicy::get_operator_pretty_name(const std::string& ope, std::string& pretty_name)
+{
+    std::list<Policies::validatorType>::const_iterator it = Policies::existing_validator.begin();
+
+    for (; it != Policies::existing_validator.end(); ++it)
+    {
+        if (it->name == ope)
+        {
+            pretty_name = it->pretty_name;
+            return 0;
+        }
+    }
+
+    return -1;
+}
+
 }
