@@ -11,6 +11,7 @@
 
 //---------------------------------------------------------------------------
 #include <algorithm>
+#include <map>
 #include <ZenLib/Ztring.h>
 #include "Common/Httpd.h"
 #include "Common/LibEventHttpd.h"
@@ -37,7 +38,7 @@ namespace MediaConch
     //**************************************************************************
     // Daemon
     //**************************************************************************
-    std::string Daemon::version = "16.01.0";
+    std::string Daemon::version = "16.03.0";
 
     //--------------------------------------------------------------------------
     Daemon::Daemon() : is_daemon(true), httpd(NULL), logger(NULL)
@@ -66,10 +67,6 @@ namespace MediaConch
         if (!MCL->get_implementation_schema_file().length())
             MCL->create_default_implementation_schema();
 
-        // If no Implementation verbosity registered, use one by default
-        if (!MCL->get_implementation_verbosity().length())
-            MCL->set_implementation_verbosity("5");
-
         httpd = new LibEventHttpd(this);
         int port = -1;
         std::string address;
@@ -89,6 +86,9 @@ namespace MediaConch
         httpd->commands.report_cb = on_report_command;
         httpd->commands.retry_cb = on_retry_command;
         httpd->commands.clear_cb = on_clear_command;
+        httpd->commands.list_cb = on_list_command;
+        httpd->commands.validate_cb = on_validate_command;
+        httpd->commands.file_from_id_cb = on_file_from_id_command;
         return 0;
     }
 
@@ -123,6 +123,13 @@ namespace MediaConch
         if (!httpd)
             return -1;
 
+        if (httpd->bind() < 0)
+        {
+            std::clog << httpd->get_error() << std::endl;
+            return -1;
+        }
+
+        std::clog << get_date() << "Daemon is running on " << httpd->get_address() << ":" << httpd->get_port() << std::endl;
         if (httpd->start() < 0)
         {
             std::clog << httpd->get_error() << std::endl;
@@ -137,7 +144,7 @@ namespace MediaConch
         if (httpd)
             httpd->finish();
         MCL->close();
-        std::clog << "Daemon is stopped" << std::endl;
+        std::clog << get_date() << "Daemon is stopped" << std::endl;
         return 0;
     }
 
@@ -172,6 +179,11 @@ namespace MediaConch
             last_argument="--configuration=";
             return DAEMON_RETURN_NONE;
         }
+        if (argument=="-pc")
+        {
+            last_argument="--pluginsconfiguration=";
+            return DAEMON_RETURN_NONE;
+        }
         if (argument=="-i")
         {
             last_argument = "--implementationschema=";
@@ -197,6 +209,7 @@ namespace MediaConch
         OPTION("--version",                 version)
         OPTION("--fork",                    fork)
         OPTION("--configuration",           configuration)
+        OPTION("--pluginsconfiguration",    plugins_configuration)
         OPTION("--compression",             compression)
         OPTION("--implementationschema",    implementationschema)
         OPTION("--implementationverbosity", implementationverbosity)
@@ -252,6 +265,21 @@ namespace MediaConch
         return DAEMON_RETURN_NONE;
     }
 
+    //--------------------------------------------------------------------------
+    int Daemon::parse_plugins_configuration(const std::string& argument)
+    {
+        size_t equal_pos = argument.find('=');
+        if (equal_pos == std::string::npos)
+        {
+            Help();
+            return DAEMON_RETURN_ERROR;
+        }
+
+        std::string file = argument.substr(equal_pos + 1);
+        MCL->set_plugins_configuration_file(file);
+        return DAEMON_RETURN_NONE;
+    }
+
     //--------------------------------------------------------------------------
     int Daemon::parse_compression(const std::string& argument)
     {
@@ -461,13 +489,39 @@ namespace MediaConch
                 continue;
             }
 
-            RESTAPI::Status_Ok *ok = new RESTAPI::Status_Ok;
+            MediaConchLib::report report_kind;
             double percent_done = 0.0;
-            bool is_done = d->MCL->is_done(*d->current_files[id], percent_done);
-            ok->id = id;
-            ok->finished = is_done;
-            if (!is_done)
+            int is_done = d->MCL->is_done(*d->current_files[id], percent_done, report_kind);
+
+            if (is_done < 0)
             {
+                RESTAPI::Status_Nok *nok = new RESTAPI::Status_Nok;
+                nok->id = id;
+                nok->error = RESTAPI::NO_REASON;
+                res.nok.push_back(nok);
+                continue;
+            }
+            RESTAPI::Status_Ok *ok = new RESTAPI::Status_Ok;
+
+            ok->id = id;
+            if (is_done == MediaConchLib::errorHttp_TRUE)
+            {
+                ok->finished = true;
+                ok->has_tool = false;
+                if (report_kind == MediaConchLib::report_MediaVeraPdf)
+                {
+                    ok->has_tool = true;
+                    ok->tool = RESTAPI::VERAPDF;
+                }
+                else if (report_kind == MediaConchLib::report_MediaDpfManager)
+                {
+                    ok->has_tool = true;
+                    ok->tool = RESTAPI::DPFMANAGER;
+                }
+            }
+            else
+            {
+                ok->finished = false;
                 ok->has_percent = true;
                 ok->done = percent_done;
             }
@@ -514,6 +568,10 @@ namespace MediaConch
                 report_set.set(MediaConchLib::report_MediaConch);
             if (req->reports[j] == RESTAPI::POLICY)
                 has_policy = true;
+            if (req->reports[j] == RESTAPI::VERAPDF)
+                report_set.set(MediaConchLib::report_MediaVeraPdf);
+            if (req->reports[j] == RESTAPI::DPFMANAGER)
+                report_set.set(MediaConchLib::report_MediaDpfManager);
         }
 
         if (!report_set.count() && !has_policy)
@@ -521,6 +579,7 @@ namespace MediaConch
 
         bool has_valid = false;
         bool valid = true;
+        std::vector<std::string> files;
         for (size_t i = 0; i < req->ids.size(); ++i)
         {
             int id = req->ids[i];
@@ -533,9 +592,10 @@ namespace MediaConch
                 continue;
             }
 
+            MediaConchLib::report report_kind;
             double percent_done = 0.0;
-            bool is_done = d->MCL->is_done(*d->current_files[id], percent_done);
-            if (!is_done)
+            int is_done = d->MCL->is_done(*d->current_files[id], percent_done, report_kind);
+            if (is_done != MediaConchLib::errorHttp_TRUE)
             {
                 RESTAPI::Report_Nok *nok = new RESTAPI::Report_Nok;
                 nok->id = id;
@@ -544,21 +604,29 @@ namespace MediaConch
                 continue;
             }
 
-            // Output
-            std::vector<std::string> files;
             files.push_back(*d->current_files[id]);
+        }
 
-            MediaConchLib::ReportRes result;
-            d->MCL->get_report(report_set, format, files,
-                              req->policies_names, req->policies_contents,
-                              &result, display_name, display_content);
-            res.ok.report = result.report;
-            if (result.has_valid)
-            {
-                has_valid = true;
-                if (!result.valid)
-                    valid = false;
-            }
+        std::stringstream verbosity;
+        if (req->has_verbosity && req->verbosity != -1)
+            verbosity << req->verbosity;
+        else
+            verbosity << d->MCL->get_implementation_verbosity();
+
+        std::map<std::string, std::string> options;
+        options["verbosity"] = verbosity.str();
+
+        // Output
+        MediaConchLib::ReportRes result;
+        d->MCL->get_report(report_set, format, files,
+                           req->policies_names, req->policies_contents,
+                           options, &result, display_name, display_content);
+        res.ok.report = result.report;
+        if (result.has_valid)
+        {
+            has_valid = true;
+            if (!result.valid)
+                valid = false;
         }
         res.ok.has_valid = has_valid;
         res.ok.valid = valid;
@@ -640,6 +708,124 @@ namespace MediaConch
         return 0;
     }
 
+    //--------------------------------------------------------------------------
+    int Daemon::on_list_command(const RESTAPI::List_Req* req, RESTAPI::List_Res& res, void *arg)
+    {
+        Daemon *d = (Daemon*)arg;
+
+        if (!d || !req)
+            return -1;
+
+        std::clog << d->get_date() << "Daemon received a list command" << std::endl;
+        std::vector<std::string> vec;
+        d->MCL->list(vec);
+        for (size_t i = 0; i < vec.size(); ++i)
+        {
+            RESTAPI::List_File *file = new RESTAPI::List_File;
+            file->file = vec[i];
+
+            size_t id = 0;
+            if (!d->file_is_registered(vec[i], id))
+            {
+                id = d->get_first_free_slot();
+                d->current_files[id] = new std::string(vec[i]);
+            }
+
+            file->id = id;
+            res.files.push_back(file);
+        }
+        std::clog << d->get_date() << "Daemon send list result: " << res.to_str() << std::endl;
+        return 0;
+    }
+
+    //--------------------------------------------------------------------------
+    int Daemon::on_validate_command(const RESTAPI::Validate_Req* req, RESTAPI::Validate_Res& res, void *arg)
+    {
+        Daemon *d = (Daemon*)arg;
+
+        if (!d || !req)
+            return -1;
+
+        std::clog << d->get_date() << "Daemon received a validate command: ";
+        std::clog << req->to_str() << std::endl;
+
+        MediaConchLib::report report;
+        if (req->report == RESTAPI::IMPLEMENTATION)
+            report = MediaConchLib::report_MediaConch;
+        else if (req->report == RESTAPI::VERAPDF)
+            report = MediaConchLib::report_MediaVeraPdf;
+        else if (req->report == RESTAPI::DPFMANAGER)
+            report = MediaConchLib::report_MediaDpfManager;
+        else if (req->report != RESTAPI::POLICY)
+            return -1;
+        else
+            report = MediaConchLib::report_Max;
+
+        std::map<std::string, int> saved_ids;
+        std::vector<std::string> files;
+        for (size_t i = 0; i < req->ids.size(); ++i)
+        {
+            int id = req->ids[i];
+            if (!d->id_is_existing(id))
+            {
+                RESTAPI::Validate_Nok *nok = new RESTAPI::Validate_Nok;
+                nok->id = id;
+                nok->error = RESTAPI::ID_NOT_EXISTING;
+                res.nok.push_back(nok);
+                continue;
+            }
+
+            MediaConchLib::report report_kind;
+            double percent_done = 0.0;
+            int is_done = d->MCL->is_done(*d->current_files[id], percent_done, report_kind);
+            if (is_done != MediaConchLib::errorHttp_TRUE)
+            {
+                RESTAPI::Validate_Nok *nok = new RESTAPI::Validate_Nok;
+                nok->id = id;
+                nok->error = RESTAPI::NOT_READY;
+                res.nok.push_back(nok);
+                continue;
+            }
+
+            // Output
+            files.push_back(*d->current_files[id]);
+            saved_ids[*d->current_files[id]] = id;
+        }
+
+        std::vector<MediaConchLib::ValidateRes*> result;
+        if (d->MCL->validate(report, files, req->policies_names, req->policies_contents,
+                             result))
+            return -1;
+
+        for (size_t i = 0; i < result.size(); ++i)
+        {
+            RESTAPI::Validate_Ok* ok = new RESTAPI::Validate_Ok;
+            ok->id = saved_ids[result[i]->file];
+            ok->valid = result[i]->valid;
+            res.ok.push_back(ok);
+        }
+        std::clog << d->get_date() << "Daemon send validate result: " << res.to_str() << std::endl;
+        return 0;
+    }
+
+    //--------------------------------------------------------------------------
+    int Daemon::on_file_from_id_command(const RESTAPI::File_From_Id_Req* req, RESTAPI::File_From_Id_Res& res, void *arg)
+    {
+        Daemon *d = (Daemon*)arg;
+
+        if (!d || !req)
+            return -1;
+
+        std::clog << d->get_date() << "Daemon received a file_from_id command" << std::endl;
+        std::clog << req->to_str() << std::endl;
+
+        if (d->id_is_existing(req->id))
+            res.file = *d->current_files[req->id];
+
+        std::clog << d->get_date() << "Daemon send file_from_id result: " << res.to_str() << std::endl;
+        return 0;
+    }
+
     //--------------------------------------------------------------------------
     size_t Daemon::get_first_free_slot()
     {
@@ -671,4 +857,16 @@ namespace MediaConch
         out << "[" << str << "]";
         return out.str();
     }
+
+    //--------------------------------------------------------------------------
+    bool Daemon::file_is_registered(const std::string& file, size_t& id)
+    {
+        for (size_t i = 0; i < current_files.size(); ++i)
+            if (current_files[i] && !current_files[i]->compare(file))
+            {
+                id = i;
+                return true;
+            }
+        return false;
+    }
 }
