@@ -8,14 +8,19 @@
 #include "ui_mainwindow.h"
 #include "menumainwindow.h"
 #include "checkerwindow.h"
+#include "resultwindow.h"
 #include "policieswindow.h"
 #include "displaywindow.h"
 #include "helpwindow.h"
+#include "verbosityspinbox.h"
+#include "Common/ImplementationReportDisplayHtmlXsl.h"
+#include "Common/FileRegistered.h"
 
 #include <QStringList>
 #include <QTextEdit>
 #include <QVBoxLayout>
 #include <QActionGroup>
+#include <QSpinBox>
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QFileDialog>
@@ -50,21 +55,27 @@ namespace MediaConch {
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
-    ui(new Ui::MainWindow)
+    ui(new Ui::MainWindow),
+    workerfiles(this)
 {
     ui->setupUi(this);
 
     MCL.init();
+
+    // Local database
+    workerfiles.create_and_configure_database();
+
     // Core configuration
     if (!MCL.get_implementation_schema_file().length())
         MCL.create_default_implementation_schema();
-    // Verbosity option
+
     if (!MCL.get_implementation_verbosity().length())
-        MCL.set_implementation_verbosity("5");
+        MCL.set_implementation_verbosity("-1");
 
     // Groups
     QActionGroup* ToolGroup = new QActionGroup(this);
     ToolGroup->addAction(ui->actionChecker);
+    ToolGroup->addAction(ui->actionResult);
     ToolGroup->addAction(ui->actionPolicies);
     ToolGroup->addAction(ui->actionDisplay);
     
@@ -72,9 +83,11 @@ MainWindow::MainWindow(QWidget *parent) :
     Layout=(QVBoxLayout*)ui->centralWidget->layout();
     Layout->setContentsMargins(0, 0, 0, 0);
     MenuView = new MenuMainWindow(this);
-    MainView=NULL;
+    checkerView=NULL;
+    resultView=NULL;
     policiesView = NULL;
     displayView = NULL;
+    verbosity = NULL;
 
     // Window
     setWindowIcon(QIcon(":/icon/icon.png"));
@@ -96,15 +109,24 @@ MainWindow::MainWindow(QWidget *parent) :
 
     // Status bar
     statusBar()->show();
-    status_msg = new QLineEdit(statusBar());
-    status_msg->setReadOnly(true);
-    statusBar()->addWidget(status_msg, 2);
     clear_msg_in_status_bar();
+
+    // Connect the signal
+    connect(this, SIGNAL(setResultView()), this, SLOT(on_actionResult_triggered()));
+    connect(this, SIGNAL(status_bar_show_message(const QString&, int)), statusBar(), SLOT(showMessage(const QString&, int)));
+    workerfiles.fill_registered_files_from_db();
+    workerfiles.start();
 }
 
 MainWindow::~MainWindow()
 {
+    workerfiles.quit();
+    workerfiles.wait();
+    if (verbosity)
+        delete verbosity;
     delete ui;
+    if (checkerView)
+        delete checkerView;
 }
 
 //***************************************************************************
@@ -112,9 +134,30 @@ MainWindow::~MainWindow()
 //***************************************************************************
 
 //---------------------------------------------------------------------------
-void MainWindow::add_file_to_list(const QString& file)
+void MainWindow::add_file_to_list(const QString& file, const QString& path,
+                                  const QString& policy, const QString& display,
+                                  const QString& v)
 {
-    files.push_back(file.toStdString());
+    std::string filename = std::string(file.toUtf8().data(), file.toUtf8().length());
+    std::string filepath = std::string(path.toUtf8().data(), path.toUtf8().length());
+    int policy_i = policy.toInt();
+    int display_i = display.toInt();
+
+    QString verbosity(v);
+    if (verbosity == "-1")
+        verbosity = QString().fromStdString(MCL.get_implementation_verbosity());
+    int verbosity_i = -1;
+    if (verbosity.length())
+        verbosity_i = verbosity.toInt();
+
+    workerfiles.add_file_to_list(filename, filepath, policy_i, display_i, verbosity_i);
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::remove_file_to_list(const QString& file)
+{
+    std::string filename = std::string(file.toUtf8().data(), file.toUtf8().length());
+    workerfiles.remove_file_registered_from_file(filename);
 }
 
 void MainWindow::policy_to_delete(int index)
@@ -133,10 +176,10 @@ void MainWindow::Run()
     switch (current_view)
     {
         case RUN_CHECKER_VIEW:
-            //TODO: fill the view if file already here
-            // if (!files.empty())
-            //     C.Run();
-            createWebView();
+            createCheckerView();
+            break;
+        case RUN_RESULT_VIEW:
+            createResultView();
             break;
         case RUN_POLICIES_VIEW:
             createPoliciesView();
@@ -145,10 +188,7 @@ void MainWindow::Run()
             createDisplayView();
             break;
         default:
-            //TODO: fill the view if file already here
-            // if (!files.empty())
-            //     C.Run();
-            createWebView();
+            createCheckerView();
             break;
     }
 }
@@ -166,33 +206,9 @@ int MainWindow::transform_with_xslt_memory(const std::string& report, const std:
 }
 
 //---------------------------------------------------------------------------
-void MainWindow::checker_add_file(const QString& file, int policy)
+void MainWindow::get_registered_files(std::map<std::string, FileRegistered>& files)
 {
-    MainView->checker_add_file(file, policy);
-}
-
-//---------------------------------------------------------------------------
-void MainWindow::checker_add_files(QFileInfoList& list, int policy)
-{
-    MainView->checker_add_files(list, policy);
-}
-
-//---------------------------------------------------------------------------
-void MainWindow::checker_add_policy_file(const QString& file, QString& policy)
-{
-    MainView->checker_add_policy_file(file, policy);
-}
-
-//---------------------------------------------------------------------------
-void MainWindow::checker_add_policy_files(QFileInfoList& list, QString& policy)
-{
-    MainView->checker_add_policy_files(list, policy);
-}
-
-//---------------------------------------------------------------------------
-const std::vector<std::string>& MainWindow::policy_file_registered()
-{
-    return policies;
+    workerfiles.get_registered_files(files);
 }
 
 //---------------------------------------------------------------------------
@@ -241,6 +257,32 @@ int MainWindow::exporting_to_schematron_file(size_t pos)
     return 0;
 }
 
+//---------------------------------------------------------------------------
+int MainWindow::exporting_to_unknown_file(size_t pos)
+{
+    QString path = get_local_folder();
+    path += "/policies";
+
+    QDir dir(path);
+    if (!dir.exists())
+        dir.mkpath(dir.absolutePath());
+
+    Policy* p = MCL.get_policy(pos);
+    if (!p)
+        return -1;
+
+    path += "/" + QString().fromStdString(p->filename);
+    QString filename = QFileDialog::getSaveFileName(this, tr("Save Policy"),
+                                              path, tr("XML (*.xml)"));
+
+    if (!filename.length())
+        return -1;
+
+    std::string f = filename.toStdString();
+    MCL.save_policy(pos, &f);
+    return 0;
+}
+
 //---------------------------------------------------------------------------
 int MainWindow::exporting_to_xslt_file(size_t pos)
 {
@@ -339,31 +381,19 @@ void MainWindow::add_default_displays()
 //---------------------------------------------------------------------------
 void MainWindow::add_xslt_display(const QString& display_xslt)
 {
-    MainView->set_display_xslt(display_xslt);
+    checkerView->set_display_xslt(display_xslt);
 }
 
 //---------------------------------------------------------------------------
 void MainWindow::remove_xslt_display()
 {
-    MainView->reset_display_xslt();
-}
-
-//---------------------------------------------------------------------------
-void MainWindow::add_policy_to_list(const QString& policy)
-{
-    policies.push_back(policy.toStdString());
-}
-
-//---------------------------------------------------------------------------
-void MainWindow::clear_policy_list()
-{
-    policies.clear();
+    checkerView->reset_display_xslt();
 }
 
 //---------------------------------------------------------------------------
 void MainWindow::clear_file_list()
 {
-    files.clear();
+    workerfiles.clear_files();
 }
 
 //---------------------------------------------------------------------------
@@ -385,19 +415,25 @@ std::vector<QString>& MainWindow::get_displays()
 //---------------------------------------------------------------------------
 void MainWindow::on_actionOpen_triggered()
 {
-    QStringList list=QFileDialog::getOpenFileNames(this, "Open file", "", "Video files (*.avi *.mkv *.mov *.mxf *.mp4);;All (*.*)", 0, QFileDialog::DontUseNativeDialog);
+    QStringList list = QFileDialog::getOpenFileNames(this, "Open file", "", "Video files (*.avi *.mkv *.mov *.mxf *.mp4);;All (*.*)", 0, QFileDialog::DontUseNativeDialog);
     if (list.empty())
         return;
 
     for (int pos = 0; pos < list.size(); ++pos)
-        files.push_back(list[pos].toStdString());
+    {
+        QFileInfo file(list[pos]);
+        QString filename = file.fileName();
+        QString filepath = file.absolutePath();
+
+        add_file_to_list(filename, filepath, "-1", "-1", "-1");
+    }
 
     current_view = RUN_CHECKER_VIEW;
     Run();
 
-    if (!MainView)
+    if (!checkerView)
         return;
-    MainView->change_local_files(list);
+    checkerView->change_local_files(list);
 }
 
 //---------------------------------------------------------------------------
@@ -411,6 +447,17 @@ void MainWindow::on_actionChecker_triggered()
     Run();
 }
 
+//---------------------------------------------------------------------------
+void MainWindow::on_actionResult_triggered()
+{
+    if (!ui->actionResult->isChecked())
+        ui->actionResult->setChecked(true);
+    if (clearVisualElements() < 0)
+        return;
+    current_view = RUN_RESULT_VIEW;
+    Run();
+}
+
 //---------------------------------------------------------------------------
 void MainWindow::on_actionPolicies_triggered()
 {
@@ -442,7 +489,7 @@ void MainWindow::on_actionChooseSchema_triggered()
 
     std::string err;
     if (MCL.import_policy_from_file(file.toStdString(), err) < 0)
-        set_msg_error_to_status_bar("Policy not valid");
+        set_msg_to_status_bar("Policy not valid");
 
     if (!ui->actionPolicies->isChecked())
         ui->actionPolicies->setChecked(true);
@@ -450,6 +497,78 @@ void MainWindow::on_actionChooseSchema_triggered()
     Run();
 }
 
+//---------------------------------------------------------------------------
+void MainWindow::on_actionVerbosity_triggered()
+{
+    if (!verbosity)
+    {
+        verbosity = new VerbositySpinbox(NULL);
+        int w = width();
+        int left = 0;
+        if (w < 220)
+            w = 220;
+        else
+        {
+            w -= 220;
+            left = w / 3;
+            w = left + 220;
+        }
+
+        int h = height();
+        int up = 0;
+        if (h < 85)
+            h = 85;
+        else
+        {
+            h -= 85;
+            up = h / 3;
+            h = up + 85;
+        }
+        verbosity->move(left + x(), up + y());
+        verbosity->resize(w, h);
+    }
+    QString value;
+    if (MCL.get_implementation_verbosity().length())
+        value = QString().fromStdString(MCL.get_implementation_verbosity());
+    else
+        value = "-1";
+
+    verbosity->get_verbosity_spin()->setValue(value.toInt());
+    connect(verbosity->get_buttons_box(), SIGNAL(accepted()), this, SLOT(verbosity_accepted()));
+    connect(verbosity->get_buttons_box(), SIGNAL(rejected()), this, SLOT(verbosity_rejected()));
+    verbosity->show();
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::verbosity_accepted()
+{
+    if (!verbosity)
+        return;
+
+    QString value;
+    value.setNum(verbosity->get_verbosity_spin()->value());
+    MCL.set_implementation_verbosity(value.toStdString());
+
+    delete verbosity;
+    verbosity = NULL;
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::verbosity_rejected()
+{
+    if (!verbosity)
+        return;
+
+    delete verbosity;
+    verbosity = NULL;
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::set_result_view()
+{
+    Q_EMIT setResultView();
+}
+
 //---------------------------------------------------------------------------
 void MainWindow::closeEvent(QCloseEvent *event)
 {
@@ -525,18 +644,13 @@ void MainWindow::on_actionDataFormat_triggered()
 //---------------------------------------------------------------------------
 int MainWindow::clearVisualElements()
 {
-    if (MainView)
+    if (checkerView)
+        checkerView->hide();
+
+    if (resultView)
     {
-        if (MainView->is_analyzes_done())
-        {
-            int ret = QMessageBox::warning(this, tr("MediaConch"),
-                                           tr("All analysis results will be discarded?"),
-                                           QMessageBox::Ok | QMessageBox::Cancel);
-            if (ret == QMessageBox::Cancel)
-                return -1;
-        }
-        delete MainView;
-        MainView=NULL;
+        delete resultView;
+        resultView = NULL;
     }
 
     if (policiesView)
@@ -555,17 +669,28 @@ int MainWindow::clearVisualElements()
 }
 
 //---------------------------------------------------------------------------
-void MainWindow::createWebView()
+void MainWindow::createCheckerView()
 {
-    if (MainView)
-        return;
+    if (checkerView)
+    {
+        delete checkerView;
+        checkerView = NULL;
+    }
 
-    if (clearVisualElements())
+    if (clearVisualElements() < 0)
         return;
-    MainView = new CheckerWindow(this);
-    QObject::connect(ui->actionCloseAll, SIGNAL(triggered()),
-                     MainView, SLOT(actionCloseAllTriggered()));
-    MainView->create_web_view();
+    checkerView = new CheckerWindow(this);
+    checkerView->create_web_view();
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::createResultView()
+{
+    if (clearVisualElements() < 0)
+        return;
+    resultView = new ResultWindow(this);
+    resultView->set_default_update_timer(MCL.get_ui_poll_request());
+    resultView->display_results();
 }
 
 //---------------------------------------------------------------------------
@@ -588,41 +713,98 @@ void MainWindow::createDisplayView()
 
 void MainWindow::set_msg_to_status_bar(const QString& message)
 {
-    statusBar()->clearMessage();
-    status_msg->setText(message);
-    QTimer::singleShot(5000, this, SLOT(update_status_bar()));
-}
-
-void MainWindow::set_msg_error_to_status_bar(const QString& message)
-{
-    statusBar()->clearMessage();
-
-    QPalette pal = status_msg->palette();
-    pal.setColor(QPalette::WindowText, Qt::red);
-    pal.setColor(QPalette::Text, Qt::red);
-    status_msg->setPalette(pal);
-    status_msg->setText(message);
-
-    QTimer::singleShot(5000, this, SLOT(update_status_bar()));
+    Q_EMIT status_bar_show_message(message, 5000);
 }
 
 void MainWindow::clear_msg_in_status_bar()
 {
-    statusBar()->clearMessage();
-    QPalette pal = status_msg->palette();
-    pal.setColor(QPalette::WindowText, Qt::black);
-    status_msg->setPalette(pal);
-    status_msg->setText(QString());
+    Q_EMIT status_bar_clear_message();
 }
 
-QStatusBar *MainWindow::get_status_bar()
+//---------------------------------------------------------------------------
+void MainWindow::create_policy_options(QString& policies)
 {
-    return statusBar();
+    const std::vector<Policy *>& list = get_all_policies();
+
+    QString system_policy;
+    QString user_policy;
+    for (size_t i = 0; i < list.size(); ++i)
+    {
+        if (list[i]->filename.length() && list[i]->filename.find(":/") == 0)
+            system_policy += QString("<option value=\"%1\">%2</option>")
+                .arg((int)i).arg(QString().fromUtf8(list[i]->title.c_str(), list[i]->title.length()));
+        else
+            user_policy += QString("<option value=\"%1\">%2</option>")
+                .arg((int)i).arg(QString().fromUtf8(list[i]->title.c_str(), list[i]->title.length()));
+    }
+
+    // Create default policy opt-group
+    if (system_policy.length())
+        policies += QString("<optgroup label=\"System policies\">%1</optgroup>").arg(system_policy);
+
+    // Create default policy opt-group
+    if (user_policy.length())
+        policies += QString("<optgroup label=\"User policies\">%1</optgroup>").arg(user_policy);
 }
 
-void MainWindow::update_status_bar()
+//---------------------------------------------------------------------------
+void MainWindow::create_displays_options(QString& displays)
 {
-    clear_msg_in_status_bar();
+    QString system_display;
+    QString user_display;
+    for (size_t i = 0; i < displays_list.size(); ++i)
+    {
+        QFileInfo file(displays_list[i]);
+        if (displays_list[i].startsWith(":/"))
+            system_display += QString("<option value=\"%1\">%2</option>")
+                .arg((int)i).arg(file.baseName());
+        else
+            user_display += QString("<option value=\"%1\">%2</option>")
+                .arg((int)i).arg(file.baseName());
+    }
+
+    // Create default display opt-group
+    if (system_display.length())
+        displays += QString("<optgroup label=\"System displays\">%1</optgroup>").arg(system_display);
+
+    // Create user display opt-group
+    if (user_display.length())
+        displays += QString("<optgroup label=\"User displays\">%1</optgroup>").arg(user_display);
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::add_policy_to_html_selection(QString& policies, QString& html, const QString& selector)
+{
+    QRegExp reg("<option selected=\"selected\" value=\"-1\">[\\n\\r\\t\\s]*Choose a policy[\\n\\r\\t\\s]*</option>");
+    int pos = html.indexOf(selector);
+
+    reg.setMinimal(true);
+
+    if (pos == -1)
+        return;
+
+    if ((pos = reg.indexIn(html, pos)) != -1)
+    {
+        pos += reg.matchedLength();
+        html.insert(pos, policies);
+    }
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::add_display_to_html_selection(QString& displays, QString& html, const QString& selector)
+{
+    QRegExp reg("<option selected=\"selected\" value=\"-1\">[\\n\\r\\t\\s]*Choose a Display[\\n\\r\\t\\s]*</option>");
+    reg.setMinimal(true);
+
+    int pos = html.indexOf(selector);
+    if (pos == -1)
+        return;
+
+    if ((pos = reg.indexIn(html, pos)) != -1)
+    {
+        pos += reg.matchedLength();
+        html.insert(pos, displays);
+    }
 }
 
 //***************************************************************************
@@ -647,6 +829,12 @@ void MainWindow::checker_selected()
     on_actionChecker_triggered();
 }
 
+//---------------------------------------------------------------------------
+void MainWindow::result_selected()
+{
+    on_actionResult_triggered();
+}
+
 //---------------------------------------------------------------------------
 void MainWindow::policies_selected()
 {
@@ -714,30 +902,30 @@ int MainWindow::is_analyze_finished(const std::vector<std::string>& files, doubl
 }
 
 //---------------------------------------------------------------------------
-QString MainWindow::get_implementationreport_xml(const std::string& file,
-                                                 const std::string& display_name,
-                                                 const std::string& display_content,
-                                                 bool& is_valid)
+int MainWindow::is_analyze_finished(const std::string& file, double& percent_done, MediaConchLib::report& report_kind)
 {
-    const std::string* dname = display_name.length() ? &display_name : NULL;
-    const std::string* dcontent = display_content.length() ? &display_content : NULL;
+    return MCL.is_done(file, percent_done, report_kind);
+}
 
-    std::bitset<MediaConchLib::report_Max> report_set;
-    report_set.set(MediaConchLib::report_MediaConch);
+//---------------------------------------------------------------------------
+int MainWindow::validate(MediaConchLib::report report, const std::vector<std::string>& files,
+                         const std::vector<std::string>& policies_names,
+                         const std::vector<std::string>& policies_contents,
+                         std::vector<MediaConchLib::ValidateRes*>& result)
+{
+    return MCL.validate(report, files, policies_names, policies_contents, result);
+}
+
+//---------------------------------------------------------------------------
+int MainWindow::validate(MediaConchLib::report report, const std::string& file,
+                         const std::vector<std::string>& policies_names,
+                         const std::vector<std::string>& policies_contents,
+                         std::vector<MediaConchLib::ValidateRes*>& result)
+{
     std::vector<std::string> files;
     files.push_back(file);
 
-    MediaConchLib::ReportRes result;
-    std::vector<std::string> vec;
-    MCL.get_report(report_set, MediaConchLib::format_Xml, files,
-                   vec, vec,
-                   &result, dname, dcontent);
-    if (result.has_valid)
-        is_valid = result.valid;
-    else
-        is_valid = false;
-
-    return QString().fromUtf8(result.report.c_str(), result.report.length());
+    return MCL.validate(report, files, policies_names, policies_contents, result);
 }
 
 //---------------------------------------------------------------------------
@@ -754,8 +942,10 @@ QString MainWindow::get_mediainfo_and_mediatrace_xml(const std::string& file,
 
     MediaConchLib::ReportRes result;
     std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
     MCL.get_report(report_set, MediaConchLib::format_Xml, files,
                    vec, vec,
+                   options,
                    &result, &display_name, &display_content);
     return QString().fromStdString(result.report);
 }
@@ -773,8 +963,10 @@ QString MainWindow::get_mediainfo_xml(const std::string& file,
 
     MediaConchLib::ReportRes result;
     std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
     MCL.get_report(report_set, MediaConchLib::format_Xml, files,
                    vec, vec,
+                   options,
                    &result, &display_name, &display_content);
     return QString().fromUtf8(result.report.c_str(), result.report.length());
 }
@@ -790,8 +982,9 @@ QString MainWindow::get_mediainfo_jstree(const std::string& file)
 
     MediaConchLib::ReportRes result;
     std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
     MCL.get_report(report_set, MediaConchLib::format_JsTree, files,
-                   vec, vec, &result);
+                   vec, vec, options, &result);
     return QString().fromUtf8(result.report.c_str(), result.report.length());
 }
 
@@ -808,8 +1001,10 @@ QString MainWindow::get_mediatrace_xml(const std::string& file,
 
     MediaConchLib::ReportRes result;
     std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
     MCL.get_report(report_set, MediaConchLib::format_Xml, files,
                    vec, vec,
+                   options,
                    &result, &display_name, &display_content);
     return QString().fromUtf8(result.report.c_str(), result.report.length());
 }
@@ -825,35 +1020,93 @@ QString MainWindow::get_mediatrace_jstree(const std::string& file)
 
     MediaConchLib::ReportRes result;
     std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
     MCL.get_report(report_set, MediaConchLib::format_JsTree, files,
                    vec, vec,
+                   options,
                    &result);
     return QString().fromUtf8(result.report.c_str(), result.report.length());
 }
 
 //---------------------------------------------------------------------------
-bool MainWindow::validate_policy(const std::string& file, int policy,
-                                 const std::string& display_name,
-                                 const std::string& display_content,
-                                 std::string& report)
+void MainWindow::get_implementation_report(const std::string& file, QString& report, int *display_p, int *verbosity)
 {
+    FileRegistered *fr = get_file_registered_from_file(file);
+    if (!fr)
+        return;
+
+    if (!fr->analyzed)
+    {
+        delete fr;
+        return;
+    }
+
+    std::string display_content;
+    std::string display_name;
+    const std::string* dname = NULL;
+    const std::string* dcontent = NULL;
+    fill_display_used(display_p, display_name, display_content, dname, dcontent, fr);
+
+    std::bitset<MediaConchLib::report_Max> report_set;
+    report_set.set(fr->report_kind);
+
+    std::vector<std::string> files;
+    files.push_back(file);
+
+    MediaConchLib::ReportRes result;
+    std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
+    fill_options_for_report(options, verbosity);
+    MCL.get_report(report_set, MediaConchLib::format_Xml, files,
+                   vec, vec,
+                   options,
+                   &result, dname, dcontent);
+
+    report = QString().fromUtf8(result.report.c_str(), result.report.length());
+    delete fr;
+}
+
+//---------------------------------------------------------------------------
+int MainWindow::validate_policy(const std::string& file, QString& report, int policy, int *display_p)
+{
+    FileRegistered* fr = get_file_registered_from_file(file);
+    if (!fr)
+        return -1;
+
+    if (!fr->analyzed)
+    {
+        delete fr;
+        return -1;
+    }
+
     if (policy == -1)
-        return validate_policies(file, display_name, display_content, report);
+    {
+        if (fr->policy == -1)
+        {
+            delete fr;
+            return -1;
+        }
+        policy = fr->policy;
+    }
 
     Policy* p = get_policy((size_t)policy);
     if (!p)
     {
-        report = "Policy not found";
-        return false;
+        report = QString("Policy not found");
+        delete fr;
+        return 1;
     }
     std::string policy_content;
     p->dump_schema(policy_content);
 
-    const std::string* dname = display_name.length() ? &display_name : NULL;
-    const std::string* dcontent = display_content.length() ? &display_content : NULL;
-
+    std::string display_content;
+    std::string display_name;
+    const std::string* dname = NULL;
+    const std::string* dcontent = NULL;
+    fill_display_used(display_p, display_name, display_content, dname, dcontent, fr);
     std::vector<std::string> files;
     files.push_back(file);
+    delete fr;
 
     MediaConchLib::ReportRes result;
     std::bitset<MediaConchLib::report_Max> report_set;
@@ -861,43 +1114,121 @@ bool MainWindow::validate_policy(const std::string& file, int policy,
     policies_contents.push_back(policy_content);
 
     std::vector<std::string> vec;
+    std::map<std::string, std::string> options;
 
     if (MCL.get_report(report_set, MediaConchLib::format_Xml, files,
                        vec, policies_contents,
+                       options,
                        &result, dname, dcontent) < 0)
-        return false;
+        return 0;
 
-    report = result.report;
+    report = QString().fromStdString(result.report);
 
     if (!result.has_valid)
-        return true;
-    return result.valid;
+        return 1;
+    if (result.valid)
+        return 1;
+    return 0;
 }
 
 //---------------------------------------------------------------------------
-bool MainWindow::validate_policies(const std::string& file, const std::string& display_name,
-                                   const std::string& display_content,
-                                   std::string& report)
+FileRegistered* MainWindow::get_file_registered_from_file(const std::string& file)
 {
-    const std::string* dname = display_name.length() ? &display_name : NULL;
-    const std::string* dcontent = display_content.length() ? &display_content : NULL;
+    return workerfiles.get_file_registered_from_file(file);
+}
 
-    std::vector<std::string> files;
-    files.push_back(file);
+//---------------------------------------------------------------------------
+void MainWindow::remove_file_registered_from_file(const std::string& file)
+{
+    workerfiles.remove_file_registered_from_file(file);
+}
 
-    MediaConchLib::ReportRes result;
-    std::bitset<MediaConchLib::report_Max> report_set;
-    std::vector<std::string> vec;
-    if (MCL.get_report(report_set, MediaConchLib::format_Xml, files,
-                       policies, vec,
-                       &result, dname, dcontent) < 0)
-        return false;
+//---------------------------------------------------------------------------
+void MainWindow::fill_display_used(int *display_p, std::string&, std::string& display_content,
+                                   const std::string*& dname, const std::string*& dcontent,
+                                   FileRegistered* fr)
+{
+    if (fr && (fr->report_kind == MediaConchLib::report_MediaVeraPdf ||
+               fr->report_kind == MediaConchLib::report_MediaDpfManager))
+    {
+        dname = NULL;
+        dcontent = NULL;
+        return;
+    }
 
-    report = result.report;
+    if (display_p)
+    {
+        if (*display_p >= 0 && (size_t)*display_p < displays_list.size())
+        {
+            QFile display_xsl(displays_list[*display_p]);
+            display_xsl.open(QIODevice::ReadOnly | QIODevice::Text);
+            QByteArray xsl = display_xsl.readAll();
+            display_xsl.close();
+            display_content = QString(xsl).toStdString();
+        }
+        else
+            display_content = std::string(implementation_report_display_html_xsl);
+    }
+    else if (fr && fr->display > 0 && (size_t)fr->display < displays_list.size())
+    {
+        QFile display_xsl(displays_list[fr->display]);
+        display_xsl.open(QIODevice::ReadOnly | QIODevice::Text);
+        QByteArray xsl = display_xsl.readAll();
+        display_xsl.close();
+        display_content = QString(xsl).toStdString();
+    }
+    else
+        display_content = std::string(implementation_report_display_html_xsl);
+    dcontent = &display_content;
+    dname = NULL;
+}
 
-    if (!result.has_valid)
-        return true;
-    return result.valid;
+//---------------------------------------------------------------------------
+void MainWindow::set_error_http(MediaConchLib::errorHttp code)
+{
+    QString error_msg;
+    switch (code)
+    {
+        case MediaConchLib::errorHttp_INVALID_DATA:
+            error_msg = "Data sent to the daemon is not correct";
+            break;
+        case MediaConchLib::errorHttp_INIT:
+            error_msg = "Cannot initialize the HTTP connection";
+            break;
+        case MediaConchLib::errorHttp_CONNECT:
+            error_msg = "Cannot connect to the daemon";
+            break;
+        default:
+            error_msg = "Error not known";
+            break;
+    }
+    set_msg_to_status_bar(error_msg);
+}
+
+//---------------------------------------------------------------------------
+int MainWindow::get_ui_database_path(std::string& path)
+{
+    return MCL.get_ui_database_path(path);
+}
+
+//---------------------------------------------------------------------------
+void MainWindow::fill_options_for_report(std::map<std::string, std::string>& opts, int *verbosity_p)
+{
+    std::string verbosity = MCL.get_implementation_verbosity();
+
+    if (verbosity_p)
+    {
+        QString value = QString().setNum(*verbosity_p);
+        verbosity = value.toStdString();
+    }
+
+    if (verbosity.length())
+    {
+        if (verbosity == "-1")
+            opts["verbosity"] = "5";
+        else
+            opts["verbosity"] = verbosity;
+    }
 }
 
 }
